class Controls {
    constructor(drone) {
        this.drone = drone;
        this.keys = {};
        
        // Initialize rotations
        this.targetRotation = new THREE.Vector3(0, 0, 0);
        this.currentRotation = new THREE.Vector3(0, 0, 0);
        this.yawAngle = 0; // Track total yaw rotation
        
        // Initialize movement state
        this.movementIntensity = 0;
        this.maxMovementIntensity = 1.0;
        
        document.addEventListener('keydown', this.onKeyDown.bind(this));
        document.addEventListener('keyup', this.onKeyUp.bind(this));
    }

    onKeyDown(event) {
        const key = event.key.toLowerCase();
        this.keys[key] = true;
        
        // Handle Shift key specifically
        if (event.key === 'Shift') {
            this.keys['shift'] = true;
        }
        
        // Boost activation with 'b' key
        if (key === 'b') {
            this.drone.activateBoost();
        }
    }

    onKeyUp(event) {
        const key = event.key.toLowerCase();
        this.keys[key] = false;
        
        // Handle Shift key specifically
        if (event.key === 'Shift') {
            this.keys['shift'] = false;
        }
        
        // Reset movement intensity when movement keys are released
        if (['w', 's', 'a', 'd'].includes(key)) {
            this.movementIntensity = 0;
        }
    }

    update() {
        // Keep existing rotation angles (in radians) as they're working well
        const MIN_TILT = (10 * Math.PI) / 180;  // 10 degrees - minimum tilt for small movements
        const MAX_TILT = (30 * Math.PI) / 180;  // 30 degrees - maximum tilt for normal flying
        const YAW_SPEED = (60 * Math.PI) / 180; // 60 degrees per second yaw rate
        const ROTATION_SPEED = 0.15; // Smoothing factor for rotation
        
        // Update movement intensity based on key holds
        const isMoving = this.keys['w'] || this.keys['s'] || this.keys['a'] || this.keys['d'];
        if (isMoving) {
            this.movementIntensity = Math.min(
                this.movementIntensity + 0.032, // Faster intensity buildup
                this.maxMovementIntensity
            );
        } else {
            this.movementIntensity = Math.max(this.movementIntensity - 0.048, 0); // Faster deceleration
        }

        // Calculate current tilt angle based on movement intensity
        const currentTiltAngle = MIN_TILT + 
            (this.movementIntensity * (MAX_TILT - MIN_TILT));

        // Reset rotations when no keys are pressed
        if (!this.keys['w'] && !this.keys['s']) {
            this.targetRotation.x *= 0.95; // Smooth return to neutral pitch
        }
        if (!this.keys['a'] && !this.keys['d']) {
            this.targetRotation.z *= 0.95; // Smooth return to neutral roll
        }

        // Thrust control with increased values
        const HOVER_THRUST = 9.8;
        const MAX_THRUST = 35; // Increased from 25
        const MIN_THRUST = -20; // Increased from -15
        const THRUST_RATE = 3; // Increased from 2
        const VERTICAL_SPEED = 25; // Increased from 15
        
        // Handle vertical movement
        if (this.keys[' ']) {
            this.drone.thrust = Math.min(this.drone.thrust + THRUST_RATE, MAX_THRUST);
            this.drone.velocity.y = VERTICAL_SPEED;
            if (!this.drone.isFlying && this.drone.mesh.position.y > 0.5) {
                this.drone.takeoff();
            }
        } else if (this.keys['shift']) {
            this.drone.thrust = Math.max(this.drone.thrust - THRUST_RATE, MIN_THRUST);
            this.drone.velocity.y = -VERTICAL_SPEED;
        } else {
            const thrustDiff = HOVER_THRUST - this.drone.thrust;
            this.drone.thrust += thrustDiff * 0.1; // Increased from 0.05 for faster hover stabilization
            this.drone.velocity.y *= 0.95;
        }

        // Movement constants with increased values
        const MOMENTUM = 0.985; // Slightly increased from 0.98 for better glide
        const BASE_ACCELERATION = 0.5; // Increased from 0.2
        const acceleration = BASE_ACCELERATION * (1 + this.movementIntensity);
        
        // Forward/Backward movement (Pitch)
        if (this.keys['w']) {
            this.targetRotation.x = -currentTiltAngle; // Pitch forward
            // Apply movement in the direction the drone is facing
            this.drone.velocity.x -= acceleration * Math.sin(this.yawAngle);
            this.drone.velocity.z -= acceleration * Math.cos(this.yawAngle);
        } else if (this.keys['s']) {
            this.targetRotation.x = currentTiltAngle; // Pitch backward
            // Apply movement in the opposite direction the drone is facing
            this.drone.velocity.x += acceleration * Math.sin(this.yawAngle);
            this.drone.velocity.z += acceleration * Math.cos(this.yawAngle);
        }

        // Left/Right movement (Roll)
        if (this.keys['a']) {
            this.targetRotation.z = currentTiltAngle; // Roll left
            // Apply movement perpendicular to the direction the drone is facing
            this.drone.velocity.x -= acceleration * Math.cos(this.yawAngle);
            this.drone.velocity.z += acceleration * Math.sin(this.yawAngle);
        } else if (this.keys['d']) {
            this.targetRotation.z = -currentTiltAngle; // Roll right
            // Apply movement perpendicular to the direction the drone is facing
            this.drone.velocity.x += acceleration * Math.cos(this.yawAngle);
            this.drone.velocity.z -= acceleration * Math.sin(this.yawAngle);
        }

        // Apply momentum with respect to current rotation
        if (!this.keys['w'] && !this.keys['s'] && !this.keys['a'] && !this.keys['d']) {
            this.drone.velocity.x *= MOMENTUM;
            this.drone.velocity.z *= MOMENTUM;
        }

        // Yaw control (rotation around Y axis)
        if (this.keys['arrowleft']) {
            this.yawAngle += YAW_SPEED * 0.016; // Rotate left
        } else if (this.keys['arrowright']) {
            this.yawAngle -= YAW_SPEED * 0.016; // Rotate right
        }

        // Smoothly interpolate pitch and roll
        this.currentRotation.x += (this.targetRotation.x - this.currentRotation.x) * ROTATION_SPEED;
        this.currentRotation.z += (this.targetRotation.z - this.currentRotation.z) * ROTATION_SPEED;
        
        // Apply yaw directly
        this.currentRotation.y = this.yawAngle;

        // Apply rotations to drone
        this.drone.mesh.rotation.x = this.currentRotation.x;
        this.drone.mesh.rotation.y = this.currentRotation.y;
        this.drone.mesh.rotation.z = this.currentRotation.z;

        // Speed limits with boost consideration - increased speeds
        const maxSpeed = this.drone.boostActive ? 65 : 45; // Increased from 45/28
        const horizontalSpeed = Math.sqrt(
            this.drone.velocity.x * this.drone.velocity.x +
            this.drone.velocity.z * this.drone.velocity.z
        );
        
        // Update audio based on total movement
        if (this.drone.audioManager) {
            const verticalSpeed = Math.abs(this.drone.velocity.y);
            const totalSpeed = Math.sqrt(
                horizontalSpeed * horizontalSpeed +
                verticalSpeed * verticalSpeed
            );
            
            // Calculate pitch based on movement direction
            const basePitch = 1.0;
            const pitchRange = 0.15; // Reduced from 0.2 for more subtle effect
            let pitchVariation = 0;
            
            // Only apply pitch variation for significant movement
            if (totalSpeed > 0.1) {
                if (Math.abs(horizontalSpeed) > Math.abs(verticalSpeed)) {
                    // Use horizontal movement pitch
                    pitchVariation = (this.drone.velocity.x !== 0 || this.drone.velocity.z !== 0) ? pitchRange : 0;
                } else {
                    // Use vertical movement pitch
                    pitchVariation = this.drone.velocity.y > 0 ? pitchRange : -pitchRange;
                }
            }
            
            const currentPitch = basePitch + pitchVariation;
            this.drone.audioManager.updateDroneSound(totalSpeed * 2, currentPitch);
        }
        
        // Progressive speed control for horizontal movement
        if (horizontalSpeed > maxSpeed) {
            const resistance = 1 - Math.min((horizontalSpeed - maxSpeed) / maxSpeed, 0.2); // Reduced from 0.3
            this.drone.velocity.x *= resistance;
            this.drone.velocity.z *= resistance;
        }

        // Apply air resistance to horizontal movement
        const airResistance = 1 - (horizontalSpeed / (maxSpeed * 3));
        const finalResistance = Math.max(airResistance, 0.985); // Increased from 0.98
        this.drone.velocity.x *= finalResistance;
        this.drone.velocity.z *= finalResistance;
    }
}