class Environment {
    constructor(scene) {
        this.scene = scene;
        this.obstacles = [];
        this.gates = [];
        this.billboards = [];
        this.powerups = [];
        this.currentLevel = 0;
        this.environmentType = 'city'; // Default environment
        this.score = 0;
        
        // Initialize environment
        this.createEnvironment(this.environmentType);
    }

    createEnvironment(type) {
        // Clear existing environment if any
        this.clearEnvironment();
        
        // Set environment type
        this.environmentType = type || 'city';
        
        // Setup lighting first
        this.setupLighting();
        
        // Create ground based on environment type
        this.createGround();
        
        // Set sky color based on environment
        this.setSkyColor();
        
        // Create natural elements based on environment
        if (this.environmentType === 'forest' || this.environmentType === 'mountains') {
            this.createClouds();
            this.createBirds();
        }
        
        // Create environment based on type
        switch(this.environmentType) {
            case 'city':
                this.createCityEnvironment();
                break;
            case 'neonCity':
                this.createNeonCityEnvironment();
                break;
            case 'snow':
                this.createSnowEnvironment();
                break;
            case 'desert':
                this.createDesertEnvironment();
                break;
            case 'forest':
                this.createForestEnvironment();
                break;
            case 'mountains':
                this.createMountainEnvironment();
                break;
            default:
                this.createCityEnvironment();
        }
        
        // Add gates for the drone to fly through
        this.createGates();
        
        // Create billboards only in city environments
        if (this.environmentType === 'city' || this.environmentType === 'neonCity') {
            this.createBillboards();
        }
        
        // Create powerups
        this.createPowerups();
    }
    
    setupLighting() {
        // Remove any existing lights
        this.scene.children = this.scene.children.filter(child => !(child instanceof THREE.Light));
        
        // Add ambient light
        const ambientLight = new THREE.AmbientLight(0xffffff, 0.6);
        this.scene.add(ambientLight);
        
        // Add directional light (sun)
        const directionalLight = new THREE.DirectionalLight(0xffffff, 0.8);
        directionalLight.position.set(100, 100, 50);
        directionalLight.castShadow = true;
        
        // Adjust shadow properties for better quality
        directionalLight.shadow.mapSize.width = 2048;
        directionalLight.shadow.mapSize.height = 2048;
        directionalLight.shadow.camera.near = 0.5;
        directionalLight.shadow.camera.far = 500;
        directionalLight.shadow.camera.left = -100;
        directionalLight.shadow.camera.right = 100;
        directionalLight.shadow.camera.top = 100;
        directionalLight.shadow.camera.bottom = -100;
        
        this.scene.add(directionalLight);
        
        // Add hemisphere light for better ambient lighting
        const hemisphereLight = new THREE.HemisphereLight(0xffffff, 0x444444, 0.6);
        this.scene.add(hemisphereLight);
        
        // Special lighting for neon city
        if (this.environmentType === 'neonCity') {
            ambientLight.intensity = 0.3;
            directionalLight.intensity = 0.4;
            hemisphereLight.intensity = 0.3;
        }
    }
    
    clearEnvironment() {
        // Remove all objects from the scene except the ground
        while (this.scene.children.length > 0) {
            const object = this.scene.children[0];
            if (object.isGround) {
                // Skip ground, we'll handle it separately
                this.scene.children.shift();
                continue;
            }
            this.scene.remove(object);
        }
        
        // Clear arrays
        this.obstacles = [];
        this.gates = [];
        this.powerups = [];
        this.clouds = [];
        this.birds = [];
        this.bonusRings = [];
        this.billboards = [];
        
        // Reset score
        this.score = 0;
        
        // Clear any ongoing animations or effects
        if (this.snowParticles) {
            this.scene.remove(this.snowParticles);
            this.snowParticles = null;
        }
        if (this.sandstormParticles) {
            this.scene.remove(this.sandstormParticles);
            this.sandstormParticles = null;
        }
    }
    
    createGround() {
        // Create ground plane with simple color instead of texture
        let groundMaterial;
        
        switch(this.environmentType) {
            case 'city':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0x555555 }); // Simple gray color
                break;
            case 'neonCity':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0x0a0a0a }); // Dark asphalt
                break;
            case 'snow':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0xffffff }); // Snow white
                break;
            case 'desert':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0xd2b48c }); // Desert sand
                break;
            case 'forest':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0x355e3b }); // Forest green
                break;
            case 'mountains':
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0x8b4513 }); // Mountain brown
                break;
            default:
                groundMaterial = new THREE.MeshLambertMaterial({ color: 0x555555 });
        }
        
        const groundGeometry = new THREE.PlaneGeometry(300, 300);
        const ground = new THREE.Mesh(groundGeometry, groundMaterial);
        ground.rotation.x = -Math.PI / 2;
        ground.position.y = -10;
        ground.receiveShadow = true;
        this.scene.add(ground);
    }
    
    setSkyColor() {
        switch(this.environmentType) {
            case 'city':
                this.scene.background = new THREE.Color(0x87CEEB); // Sky blue
                break;
            case 'neonCity':
                this.scene.background = new THREE.Color(0x000033); // Dark blue night sky
                break;
            case 'snow':
                this.scene.background = new THREE.Color(0xB0E0E6); // Powder blue winter sky
                break;
            case 'desert':
                this.scene.background = new THREE.Color(0xFFB74D); // Warm orange desert sky
                break;
            case 'forest':
                this.scene.background = new THREE.Color(0x90EE90); // Light green
                break;
            default:
                this.scene.background = new THREE.Color(0x87CEEB); // Default sky blue
        }
        
        // Add fog based on environment
        let fogColor, fogDensity;
        
        switch(this.environmentType) {
            case 'city':
                fogColor = 0xaaaaaa;
                fogDensity = 0.005;
                break;
            case 'neonCity':
                fogColor = 0x000033;
                fogDensity = 0.01;
                break;
            case 'snow':
                fogColor = 0xffffff;
                fogDensity = 0.015;
                break;
            case 'forest':
                fogColor = 0x90EE90;
                fogDensity = 0.008;
                break;
            case 'desert':
                fogColor = 0xFFB74D;
                fogDensity = 0.003;
                break;
            default:
                fogColor = 0xaaaaaa;
                fogDensity = 0.005;
        }
        
        this.scene.fog = new THREE.FogExp2(fogColor, fogDensity);
    }
    
    createCityEnvironment() {
        // Create skyscrapers with reduced density
        const buildingPositions = [];
        const roadOffset = 25; // Increased spacing between buildings
        
        // Generate building positions with reduced density
        for (let i = -1; i <= 1; i++) {
            for (let j = -1; j <= 1; j++) {
                if (i !== 0 || j !== 0) {
                    if (Math.random() > 0.3) {
                        buildingPositions.push({
                            x: i * 70 + roadOffset,
                            z: j * 70 + roadOffset,
                            height: Math.random() * 30 + 20
                        });
                    }
                }
            }
        }
        
        // Create buildings at generated positions
        buildingPositions.forEach(pos => {
            const width = Math.random() * 10 + 5;
            const height = pos.height;
            const depth = Math.random() * 10 + 5;
            
            const geometry = new THREE.BoxGeometry(width, height, depth);
            const material = new THREE.MeshPhongMaterial({ 
                color: 0x888888,
                shininess: 30
            });
            
            const building = new THREE.Mesh(geometry, material);
            building.position.set(pos.x, height/2 - 10, pos.z);
            
            building.userData = {
                isBuilding: true,
                originalColor: 0x888888,
                points: Math.floor(height * 10),
                glowIntensity: 0,
                maxGlowIntensity: 0.5,
                glowSpeed: 0.05
            };
            
            this.addBuildingWindows(building, width, height, depth);
            
            building.castShadow = true;
            building.receiveShadow = true;
            
            this.scene.add(building);
            this.obstacles.push(building);
        });

        // Add small houses
        for (let i = 0; i < 15; i++) {
            const houseGroup = new THREE.Group();
            
            // House dimensions
            const width = 4 + Math.random() * 3;
            const height = 3 + Math.random() * 2;
            const depth = 4 + Math.random() * 3;
            
            // Main house body
            const houseGeometry = new THREE.BoxGeometry(width, height, depth);
            const houseMaterial = new THREE.MeshPhongMaterial({ 
                color: Math.random() > 0.5 ? 0xE8DCC4 : 0xD4B995, // Varied house colors
                shininess: 10
            });
            const house = new THREE.Mesh(houseGeometry, houseMaterial);
            houseGroup.add(house);
            
            // Roof
            const roofGeometry = new THREE.ConeGeometry(width * 0.7, height * 0.5, 4);
            const roofMaterial = new THREE.MeshPhongMaterial({
                color: 0x8B4513,
                shininess: 5
            });
            const roof = new THREE.Mesh(roofGeometry, roofMaterial);
            roof.position.y = height * 0.75;
            roof.rotation.y = Math.PI / 4;
            houseGroup.add(roof);
            
            // Position house
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 100 + 40;
            houseGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            houseGroup.rotation.y = Math.random() * Math.PI * 2;
            this.scene.add(houseGroup);
            this.obstacles.push(houseGroup);
        }

        // Add trees
        for (let i = 0; i < 20; i++) {
            const treeGroup = new THREE.Group();
            
            // Tree trunk
            const trunkGeometry = new THREE.CylinderGeometry(0.2, 0.3, 2, 6);
            const trunkMaterial = new THREE.MeshPhongMaterial({ color: 0x4A2B0F });
            const trunk = new THREE.Mesh(trunkGeometry, trunkMaterial);
            trunk.position.y = 1;
            treeGroup.add(trunk);
            
            // Tree top
            const topGeometry = new THREE.SphereGeometry(1.5, 8, 8);
            const topMaterial = new THREE.MeshPhongMaterial({ color: 0x228B22 });
            const top = new THREE.Mesh(topGeometry, topMaterial);
            top.position.y = 3;
            top.scale.y = 1.2;
            treeGroup.add(top);
            
            // Position tree
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 120 + 30;
            treeGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            this.scene.add(treeGroup);
        }

        // Add playground
        const playgroundGroup = new THREE.Group();
        
        // Swing set
        const swingFrame = new THREE.Group();
        const poleGeometry = new THREE.CylinderGeometry(0.2, 0.2, 4, 8);
        const poleMaterial = new THREE.MeshPhongMaterial({ color: 0x666666 });
        
        // Create swing frame
        const pole1 = new THREE.Mesh(poleGeometry, poleMaterial);
        const pole2 = new THREE.Mesh(poleGeometry, poleMaterial);
        pole1.position.x = -2;
        pole2.position.x = 2;
        swingFrame.add(pole1);
        swingFrame.add(pole2);
        
        // Top bar
        const topBarGeometry = new THREE.CylinderGeometry(0.15, 0.15, 4.5, 8);
        const topBar = new THREE.Mesh(topBarGeometry, poleMaterial);
        topBar.rotation.z = Math.PI / 2;
        topBar.position.y = 1.8;
        swingFrame.add(topBar);
        
        // Swings
        const seatGeometry = new THREE.BoxGeometry(0.4, 0.1, 0.4);
        const seatMaterial = new THREE.MeshPhongMaterial({ color: 0xff0000 });
        const swing1 = new THREE.Mesh(seatGeometry, seatMaterial);
        const swing2 = new THREE.Mesh(seatGeometry, seatMaterial);
        swing1.position.set(-1, 0.5, 0);
        swing2.position.set(1, 0.5, 0);
        swingFrame.add(swing1);
        swingFrame.add(swing2);
        
        // Slide
        const slideGroup = new THREE.Group();
        const slideGeometry = new THREE.BoxGeometry(1, 0.1, 4);
        const slideMaterial = new THREE.MeshPhongMaterial({ color: 0x4488ff });
        const slide = new THREE.Mesh(slideGeometry, slideMaterial);
        slide.rotation.x = -Math.PI / 6;
        slide.position.set(0, 1.5, 1);
        slideGroup.add(slide);
        
        // Slide platform
        const platformGeometry = new THREE.BoxGeometry(2, 0.2, 2);
        const platform = new THREE.Mesh(platformGeometry, poleMaterial);
        platform.position.set(0, 3, 0);
        slideGroup.add(platform);
        
        // Add to playground
        playgroundGroup.add(swingFrame);
        playgroundGroup.add(slideGroup);
        
        // Position playground
        playgroundGroup.position.set(30, -10, 30);
        this.scene.add(playgroundGroup);

        // Add clouds
        for (let i = 0; i < 15; i++) {
            const cloudGroup = new THREE.Group();
            
            // Create puffy cloud parts
            const cloudGeometry = new THREE.SphereGeometry(2, 8, 8);
            const cloudMaterial = new THREE.MeshPhongMaterial({
                color: 0xffffff,
                transparent: true,
                opacity: 0.8
            });
            
            // Create main cloud body
            for (let j = 0; j < 5; j++) {
                const cloudPart = new THREE.Mesh(cloudGeometry, cloudMaterial);
                cloudPart.position.set(
                    Math.random() * 4 - 2,
                    Math.random() * 1.5,
                    Math.random() * 4 - 2
                );
                cloudPart.scale.set(
                    Math.random() * 0.8 + 0.6,
                    Math.random() * 0.3 + 0.3,
                    Math.random() * 0.8 + 0.6
                );
                cloudGroup.add(cloudPart);
            }
            
            // Position cloud
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 150 + 50;
            const height = Math.random() * 30 + 40;
            cloudGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            cloudGroup.userData = { isCloud: true, speed: Math.random() * 0.2 + 0.1 };
            this.scene.add(cloudGroup);
        }

        // Add birds
        for (let i = 0; i < 10; i++) {
            const birdGroup = new THREE.Group();
            
            // Bird body
            const bodyGeometry = new THREE.ConeGeometry(0.2, 1, 4);
            const bodyMaterial = new THREE.MeshPhongMaterial({ color: 0x333333 });
            const body = new THREE.Mesh(bodyGeometry, bodyMaterial);
            body.rotation.x = Math.PI / 2;
            birdGroup.add(body);
            
            // Wings
            const wingGeometry = new THREE.PlaneGeometry(1, 0.5);
            const wingMaterial = new THREE.MeshPhongMaterial({
                color: 0x333333,
                side: THREE.DoubleSide
            });
            
            const leftWing = new THREE.Mesh(wingGeometry, wingMaterial);
            const rightWing = new THREE.Mesh(wingGeometry, wingMaterial);
            leftWing.position.set(-0.5, 0, 0);
            rightWing.position.set(0.5, 0, 0);
            birdGroup.add(leftWing);
            birdGroup.add(rightWing);
            
            // Position bird
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 100 + 30;
            const height = Math.random() * 20 + 30;
            birdGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            birdGroup.userData = { 
                isBird: true, 
                speed: Math.random() * 0.3 + 0.2,
                wingSpeed: Math.random() * 0.2 + 0.1
            };
            this.scene.add(birdGroup);
        }

        // Add hot air balloons
        for (let i = 0; i < 5; i++) {
            const balloonGroup = new THREE.Group();
            
            // Balloon part
            const balloonGeometry = new THREE.SphereGeometry(3, 16, 16);
            const balloonMaterial = new THREE.MeshPhongMaterial({
                color: new THREE.Color(Math.random() * 0xffffff),
                shininess: 30
            });
            const balloon = new THREE.Mesh(balloonGeometry, balloonMaterial);
            balloon.scale.y = 1.2;
            balloonGroup.add(balloon);
            
            // Basket
            const basketGeometry = new THREE.BoxGeometry(1.5, 1, 1.5);
            const basketMaterial = new THREE.MeshPhongMaterial({ color: 0x8B4513 });
            const basket = new THREE.Mesh(basketGeometry, basketMaterial);
            basket.position.y = -4;
            balloonGroup.add(basket);
            
            // Ropes
            const ropeGeometry = new THREE.CylinderGeometry(0.02, 0.02, 4);
            const ropeMaterial = new THREE.MeshPhongMaterial({ color: 0x8B4513 });
            for (let j = 0; j < 4; j++) {
                const rope = new THREE.Mesh(ropeGeometry, ropeMaterial);
                const angle = (j * Math.PI / 2);
                rope.position.set(
                    Math.cos(angle) * 0.7,
                    -2,
                    Math.sin(angle) * 0.7
                );
                balloonGroup.add(rope);
            }
            
            // Position balloon
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 120 + 40;
            const height = Math.random() * 30 + 45;
            balloonGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            balloonGroup.userData = { 
                isBalloon: true, 
                speed: Math.random() * 0.08 + 0.03,
                heightVariation: Math.random() * 0.05
            };
            this.scene.add(balloonGroup);
        }

        // Add minimalistic track
        const trackPoints = [];
        const trackRadius = 80;
        const trackSegments = 12;
        
        for (let i = 0; i <= trackSegments; i++) {
            const angle = (i / trackSegments) * Math.PI * 2;
            trackPoints.push(
                new THREE.Vector3(
                    Math.cos(angle) * trackRadius,
                    -9.9,
                    Math.sin(angle) * trackRadius
                )
            );
        }
        
        const trackGeometry = new THREE.BufferGeometry().setFromPoints(trackPoints);
        const trackMaterial = new THREE.LineBasicMaterial({ color: 0x444444, linewidth: 2 });
        const track = new THREE.Line(trackGeometry, trackMaterial);
        this.scene.add(track);

        // Add bonus rings between buildings
        this.createBonusRings();
    }
    
    createNeonCityEnvironment() {
        // Set night sky
        this.scene.background = new THREE.Color(0x000022);
        this.scene.fog = new THREE.FogExp2(0x000033, 0.01);
        
        // Create grid layout for buildings
        const gridSize = 5;
        const spacing = 30;
        const offset = (gridSize * spacing) / 2;
        
        for (let i = 0; i < gridSize; i++) {
            for (let j = 0; j < gridSize; j++) {
                if (Math.random() > 0.3) {
                    const width = Math.random() * 8 + 4;
                    const height = Math.random() * 40 + 20;
                    const depth = Math.random() * 8 + 4;
                    
                    const geometry = new THREE.BoxGeometry(width, height, depth);
                    const material = new THREE.MeshPhongMaterial({ 
                        color: 0x222222,
                        shininess: 50
                    });
                    
                    const building = new THREE.Mesh(geometry, material);
                    
                    building.position.set(
                        i * spacing - offset + (Math.random() - 0.5) * 10,
                        height / 2 - 10,
                        j * spacing - offset + (Math.random() - 0.5) * 10
                    );
                    
                    this.addNeonWindows(building, width, height, depth);
                    
                    building.castShadow = true;
                    building.receiveShadow = true;
                    
                    this.scene.add(building);
                    this.obstacles.push(building);
                }
            }
        }

        // Add small neon houses
        for (let i = 0; i < 12; i++) {
            const houseGroup = new THREE.Group();
            
            const width = 3 + Math.random() * 2;
            const height = 2 + Math.random() * 2;
            const depth = 3 + Math.random() * 2;
            
            const houseGeometry = new THREE.BoxGeometry(width, height, depth);
            const houseMaterial = new THREE.MeshPhongMaterial({ 
                color: 0x111111,
                emissive: new THREE.Color(Math.random() > 0.5 ? 0x00ffff : 0xff00ff),
                emissiveIntensity: 0.2
            });
            const house = new THREE.Mesh(houseGeometry, houseMaterial);
            houseGroup.add(house);
            
            // Add neon trim
            const trimGeometry = new THREE.BoxGeometry(width + 0.1, 0.1, depth + 0.1);
            const trimMaterial = new THREE.MeshPhongMaterial({
                color: 0xffffff,
                emissive: new THREE.Color(Math.random() > 0.5 ? 0xff0000 : 0x00ff00),
                emissiveIntensity: 0.5
            });
            const trim = new THREE.Mesh(trimGeometry, trimMaterial);
            trim.position.y = height / 2;
            houseGroup.add(trim);
            
            // Position house
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 90 + 50;
            houseGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            houseGroup.rotation.y = Math.random() * Math.PI * 2;
            this.scene.add(houseGroup);
            this.obstacles.push(houseGroup);
        }

        // Add cyber-trees
        for (let i = 0; i < 15; i++) {
            const treeGroup = new THREE.Group();
            
            // Glowing trunk
            const trunkGeometry = new THREE.CylinderGeometry(0.2, 0.3, 3, 6);
            const trunkMaterial = new THREE.MeshPhongMaterial({ 
                color: 0x222222,
                emissive: 0x00ffff,
                emissiveIntensity: 0.3
            });
            const trunk = new THREE.Mesh(trunkGeometry, trunkMaterial);
            trunk.position.y = 1.5;
            treeGroup.add(trunk);
            
            // Glowing foliage
            const topGeometry = new THREE.ConeGeometry(1.2, 2.5, 6);
            const topMaterial = new THREE.MeshPhongMaterial({ 
                color: 0x222222,
                emissive: 0x00ff00,
                emissiveIntensity: 0.2
            });
            const top = new THREE.Mesh(topGeometry, topMaterial);
            top.position.y = 3.5;
            treeGroup.add(top);
            
            // Position tree
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 110 + 40;
            treeGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            this.scene.add(treeGroup);
        }

        // Add neon clouds
        for (let i = 0; i < 12; i++) {
            const cloudGroup = new THREE.Group();
            
            // Create cyber-cloud parts
            const cloudGeometry = new THREE.BoxGeometry(3, 1, 3);
            const cloudMaterial = new THREE.MeshPhongMaterial({
                color: 0x000000,
                emissive: new THREE.Color(0x000066),
                emissiveIntensity: 0.5,
                transparent: true,
                opacity: 0.7
            });
            
            // Create main cloud body with geometric shapes
            for (let j = 0; j < 4; j++) {
                const cloudPart = new THREE.Mesh(cloudGeometry, cloudMaterial);
                cloudPart.position.set(
                    Math.random() * 4 - 2,
                    Math.random() * 1.5,
                    Math.random() * 4 - 2
                );
                cloudPart.scale.set(
                    Math.random() * 0.8 + 0.6,
                    Math.random() * 0.3 + 0.3,
                    Math.random() * 0.8 + 0.6
                );
                cloudPart.rotation.y = Math.random() * Math.PI;
                cloudGroup.add(cloudPart);
            }
            
            // Position cloud
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 150 + 50;
            const height = Math.random() * 30 + 40;
            cloudGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            cloudGroup.userData = { 
                isNeonCloud: true, 
                speed: Math.random() * 0.15 + 0.05,
                glowSpeed: Math.random() * 0.02 + 0.01
            };
            this.scene.add(cloudGroup);
        }

        // Add cyber birds (drones)
        for (let i = 0; i < 8; i++) {
            const droneGroup = new THREE.Group();
            
            // Drone body
            const bodyGeometry = new THREE.BoxGeometry(1, 0.2, 1);
            const bodyMaterial = new THREE.MeshPhongMaterial({
                color: 0x111111,
                emissive: 0x00ffff,
                emissiveIntensity: 0.5
            });
            const body = new THREE.Mesh(bodyGeometry, bodyMaterial);
            droneGroup.add(body);
            
            // Propellers
            const propGeometry = new THREE.BoxGeometry(0.4, 0.05, 0.1);
            const propMaterial = new THREE.MeshPhongMaterial({
                color: 0x111111,
                emissive: 0xff0000,
                emissiveIntensity: 0.5
            });
            
            for (let j = 0; j < 4; j++) {
                const prop = new THREE.Mesh(propGeometry, propMaterial);
                const angle = (j * Math.PI / 2);
                prop.position.set(
                    Math.cos(angle) * 0.5,
                    0.1,
                    Math.sin(angle) * 0.5
                );
                prop.rotation.y = angle;
                droneGroup.add(prop);
            }
            
            // Position drone
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 100 + 30;
            const height = Math.random() * 20 + 25;
            droneGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            droneGroup.userData = { 
                isDrone: true, 
                speed: Math.random() * 0.4 + 0.2,
                rotationSpeed: Math.random() * 0.02 + 0.01
            };
            this.scene.add(droneGroup);
        }

        // Add neon airships
        for (let i = 0; i < 3; i++) {
            const airshipGroup = new THREE.Group();
            
            // Main body
            const bodyGeometry = new THREE.CapsuleGeometry(2, 8, 8, 8);
            const bodyMaterial = new THREE.MeshPhongMaterial({
                color: 0x111111,
                emissive: new THREE.Color(Math.random() > 0.5 ? 0xff00ff : 0x00ffff),
                emissiveIntensity: 0.3
            });
            const body = new THREE.Mesh(bodyGeometry, bodyMaterial);
            body.rotation.z = Math.PI / 2;
            airshipGroup.add(body);
            
            // Cabin
            const cabinGeometry = new THREE.BoxGeometry(1, 1, 2);
            const cabinMaterial = new THREE.MeshPhongMaterial({
                color: 0x222222,
                emissive: 0xffff00,
                emissiveIntensity: 0.3
            });
            const cabin = new THREE.Mesh(cabinGeometry, cabinMaterial);
            cabin.position.y = -1.5;
            airshipGroup.add(cabin);
            
            // Position airship
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 120 + 40;
            const height = Math.random() * 30 + 45;
            airshipGroup.position.set(
                Math.cos(angle) * radius,
                height,
                Math.sin(angle) * radius
            );
            
            airshipGroup.userData = { 
                isAirship: true, 
                speed: Math.random() * 0.08 + 0.03,
                glowPulse: Math.random() * 0.02 + 0.01
            };
            this.scene.add(airshipGroup);
        }

        // Add minimalistic neon track
        const trackPoints = [];
        const trackRadius = 70;
        const trackSegments = 16;
        
        for (let i = 0; i <= trackSegments; i++) {
            const angle = (i / trackSegments) * Math.PI * 2;
            trackPoints.push(
                new THREE.Vector3(
                    Math.cos(angle) * trackRadius,
                    -9.9,
                    Math.sin(angle) * trackRadius
                )
            );
        }
        
        const trackGeometry = new THREE.BufferGeometry().setFromPoints(trackPoints);
        const trackMaterial = new THREE.LineBasicMaterial({ 
            color: 0x00ffff,
            linewidth: 2
        });
        const track = new THREE.Line(trackGeometry, trackMaterial);
        this.scene.add(track);
    }
    
    createSnowEnvironment() {
        // Create snow-covered cabins
        for (let i = 0; i < 10; i++) {
            const cabinGroup = new THREE.Group();
            
            // Cabin dimensions
            const width = 8 + Math.random() * 4;
            const height = 6 + Math.random() * 3;
            const depth = 8 + Math.random() * 4;
            
            // Main cabin body
            const cabinGeometry = new THREE.BoxGeometry(width, height, depth);
            const cabinMaterial = new THREE.MeshPhongMaterial({ 
                color: 0x8B4513, // Wood color
                shininess: 5
            });
            const cabin = new THREE.Mesh(cabinGeometry, cabinMaterial);
            cabinGroup.add(cabin);
            
            // Roof (triangular prism)
            const roofHeight = height * 0.5;
            const roofGeometry = new THREE.CylinderGeometry(0.1, width * 0.6, roofHeight, 4, 1);
            const roofMaterial = new THREE.MeshPhongMaterial({
                color: 0x4A4A4A, // Dark gray
                shininess: 10
            });
            const roof = new THREE.Mesh(roofGeometry, roofMaterial);
            roof.rotation.z = Math.PI / 2;
            roof.rotation.y = Math.PI / 4;
            roof.position.y = height * 0.75;
            cabinGroup.add(roof);
            
            // Snow on roof
            const snowRoofGeometry = new THREE.CylinderGeometry(0.05, width * 0.5, roofHeight * 0.9, 4, 1);
            const snowMaterial = new THREE.MeshPhongMaterial({
                color: 0xFFFFFF,
                shininess: 70
            });
            const snowRoof = new THREE.Mesh(snowRoofGeometry, snowMaterial);
            snowRoof.rotation.z = Math.PI / 2;
            snowRoof.rotation.y = Math.PI / 4;
            snowRoof.position.y = height * 0.8;
            cabinGroup.add(snowRoof);
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 100 + 20;
            cabinGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            // Random rotation
            cabinGroup.rotation.y = Math.random() * Math.PI * 2;
            
            cabinGroup.castShadow = true;
            cabinGroup.receiveShadow = true;
            
            this.scene.add(cabinGroup);
            this.obstacles.push(cabinGroup);
        }
        
        // Create pine trees with snow
        for (let i = 0; i < 50; i++) {
            const treeGroup = new THREE.Group();
            
            const height = Math.random() * 12 + 8;
            const radius = height * 0.2;
            
            // Tree trunk
            const trunkGeometry = new THREE.CylinderGeometry(radius * 0.2, radius * 0.3, height * 0.3, 8);
            const trunkMaterial = new THREE.MeshPhongMaterial({ color: 0x4A2B0F });
            const trunk = new THREE.Mesh(trunkGeometry, trunkMaterial);
            trunk.position.y = height * 0.15;
            treeGroup.add(trunk);
            
            // Tree foliage layers (3-4 layers)
            const layers = Math.floor(Math.random() * 2) + 3;
            for (let j = 0; j < layers; j++) {
                const layerRadius = radius * (1.5 - j * 0.2);
                const layerHeight = height * 0.25;
                
                // Green foliage
                const foliageGeometry = new THREE.ConeGeometry(layerRadius, layerHeight, 8);
                const foliageMaterial = new THREE.MeshPhongMaterial({ 
                    color: 0x1B4D3E,
                    shininess: 5
                });
                const foliage = new THREE.Mesh(foliageGeometry, foliageMaterial);
                foliage.position.y = height * (0.3 + j * 0.2);
                treeGroup.add(foliage);
                
                // Snow layer on top
                const snowGeometry = new THREE.ConeGeometry(layerRadius * 0.9, layerHeight * 0.2, 8);
                const snowMaterial = new THREE.MeshPhongMaterial({ 
                    color: 0xFFFFFF,
                    shininess: 70
                });
                const snow = new THREE.Mesh(snowGeometry, snowMaterial);
                snow.position.y = height * (0.4 + j * 0.2);
                treeGroup.add(snow);
            }
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const positionRadius = Math.random() * 140 + 10;
            treeGroup.position.set(
                Math.cos(angle) * positionRadius,
                -10,
                Math.sin(angle) * positionRadius
            );
            
            treeGroup.castShadow = true;
            treeGroup.receiveShadow = true;
            
            this.scene.add(treeGroup);
            this.obstacles.push(treeGroup);
        }
        
        // Create ice formations
        for (let i = 0; i < 15; i++) {
            const iceGroup = new THREE.Group();
            
            // Create main ice crystal
            const height = Math.random() * 6 + 4;
            const iceGeometry = new THREE.ConeGeometry(2, height, 6);
            const iceMaterial = new THREE.MeshPhongMaterial({
                color: 0xADD8E6,
                transparent: true,
                opacity: 0.6,
                shininess: 90
            });
            
            const mainCrystal = new THREE.Mesh(iceGeometry, iceMaterial);
            mainCrystal.rotation.x = Math.PI;
            iceGroup.add(mainCrystal);
            
            // Add smaller crystals around
            for (let j = 0; j < 3; j++) {
                const smallHeight = height * (0.4 + Math.random() * 0.3);
                const smallGeometry = new THREE.ConeGeometry(1, smallHeight, 6);
                const smallCrystal = new THREE.Mesh(smallGeometry, iceMaterial);
                
                const angle = (j * Math.PI * 2) / 3;
                const distance = 1.5;
                smallCrystal.position.set(
                    Math.cos(angle) * distance,
                    -smallHeight * 0.3,
                    Math.sin(angle) * distance
                );
                smallCrystal.rotation.x = Math.PI;
                smallCrystal.rotation.y = Math.random() * Math.PI;
                
                iceGroup.add(smallCrystal);
            }
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 120 + 20;
            iceGroup.position.set(
                Math.cos(angle) * radius,
                -8,
                Math.sin(angle) * radius
            );
            
            iceGroup.castShadow = true;
            iceGroup.receiveShadow = true;
            
            this.scene.add(iceGroup);
            this.obstacles.push(iceGroup);
        }
        
        // Create snow piles
        for (let i = 0; i < 25; i++) {
            const snowPileGroup = new THREE.Group();
            
            // Create main snow pile
            const radius = 2 + Math.random() * 3;
            const height = radius * 0.6;
            const segments = 8;
            
            const snowGeometry = new THREE.SphereGeometry(radius, segments, segments);
            const snowMaterial = new THREE.MeshPhongMaterial({
                color: 0xFFFFFF,
                shininess: 50
            });
            
            const snowPile = new THREE.Mesh(snowGeometry, snowMaterial);
            snowPile.scale.y = 0.5;
            snowPileGroup.add(snowPile);
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const distance = Math.random() * 140 + 10;
            snowPileGroup.position.set(
                Math.cos(angle) * distance,
                -9,
                Math.sin(angle) * distance
            );
            
            this.scene.add(snowPileGroup);
        }
    }
    
    createDesertEnvironment() {
        console.log("Creating desert environment...");
        
        // Desert sand material with improved texture
        const sandMaterial = new THREE.MeshPhongMaterial({ 
            color: 0xD2B48C,
            shininess: 10,
            roughness: 1.0
        });
        
        // Rock material with improved texture
        const rockMaterial = new THREE.MeshPhongMaterial({ 
            color: 0x8B4513,
            shininess: 5,
            roughness: 0.8
        });
        
        // Create large sand dunes
        for (let i = 0; i < 15; i++) {
            const duneGroup = new THREE.Group();
            
            // Main dune parameters
            const width = 20 + Math.random() * 30;
            const height = 8 + Math.random() * 12;
            const depth = 15 + Math.random() * 25;
            
            // Create main dune shape using multiple segments for more natural look
            const segments = 5;
            for (let j = 0; j < segments; j++) {
                const segmentWidth = width * (1 - j * 0.15);
                const segmentHeight = height * (1 - j * 0.2);
                const duneGeometry = new THREE.ConeGeometry(
                    segmentWidth * 0.5,
                    segmentHeight,
                    8,
                    1,
                    false
                );
                
                const duneMesh = new THREE.Mesh(duneGeometry, sandMaterial);
                duneMesh.rotation.x = Math.PI;
                duneMesh.position.y = -j * (height * 0.15);
                duneGroup.add(duneMesh);
            }
            
            // Add wind-swept details
            for (let j = 0; j < 3; j++) {
                const ridgeGeometry = new THREE.CylinderGeometry(
                    0.5,
                    2,
                    width * 0.7,
                    8,
                    1
                );
                const ridge = new THREE.Mesh(ridgeGeometry, sandMaterial);
                ridge.rotation.z = Math.PI / 2;
                ridge.rotation.y = Math.random() * Math.PI * 0.25;
                ridge.position.set(
                    (Math.random() - 0.5) * width * 0.3,
                    -height * 0.3 - j * 2,
                    (Math.random() - 0.5) * depth * 0.3
                );
                duneGroup.add(ridge);
            }
            
            // Position the dune
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 120 + 30;
            duneGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            // Random rotation for variety
            duneGroup.rotation.y = Math.random() * Math.PI * 2;
            
            this.scene.add(duneGroup);
            this.obstacles.push(duneGroup);
        }
        
        // Create rock formations
        for (let i = 0; i < 8; i++) {
            const rockFormation = new THREE.Group();
            
            // Create main rock structure
            const layers = 3 + Math.floor(Math.random() * 3);
            let prevWidth = 8 + Math.random() * 6;
            let totalHeight = 0;
            
            for (let j = 0; j < layers; j++) {
                const width = prevWidth * (0.7 + Math.random() * 0.3);
                const height = 3 + Math.random() * 4;
                
                const rockGeometry = new THREE.DodecahedronGeometry(
                    width * 0.5,
                    1
                );
                
                const rock = new THREE.Mesh(rockGeometry, rockMaterial);
                rock.position.y = totalHeight;
                rock.rotation.set(
                    Math.random() * Math.PI,
                    Math.random() * Math.PI,
                    Math.random() * Math.PI
                );
                
                rockFormation.add(rock);
                
                totalHeight += height * 0.8;
                prevWidth = width;
            }
            
            // Add smaller rocks around base
            for (let j = 0; j < 5; j++) {
                const smallRockGeometry = new THREE.DodecahedronGeometry(
                    1 + Math.random() * 2,
                    0
                );
                
                const smallRock = new THREE.Mesh(smallRockGeometry, rockMaterial);
                const angle = Math.random() * Math.PI * 2;
                const distance = 4 + Math.random() * 4;
                
                smallRock.position.set(
                    Math.cos(angle) * distance,
                    Math.random() * 2,
                    Math.sin(angle) * distance
                );
                
                smallRock.rotation.set(
                    Math.random() * Math.PI,
                    Math.random() * Math.PI,
                    Math.random() * Math.PI
                );
                
                rockFormation.add(smallRock);
            }
            
            // Position the formation
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 100 + 40;
            rockFormation.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            this.scene.add(rockFormation);
            this.obstacles.push(rockFormation);
        }
        
        // Create oasis
        const oasisGroup = new THREE.Group();
        
        // Create water pool
        const poolGeometry = new THREE.CylinderGeometry(8, 8, 1, 32);
        const poolMaterial = new THREE.MeshPhongMaterial({
            color: 0x4B9CD3,
            transparent: true,
            opacity: 0.8,
            shininess: 90
        });
        const pool = new THREE.Mesh(poolGeometry, poolMaterial);
        pool.position.y = -9.5;
        oasisGroup.add(pool);
        
        // Add palm trees around oasis
        for (let i = 0; i < 6; i++) {
            const palmGroup = new THREE.Group();
            
            // Create trunk
            const trunkHeight = 8 + Math.random() * 4;
            const trunkGeometry = new THREE.CylinderGeometry(0.3, 0.5, trunkHeight, 8);
            const trunkMaterial = new THREE.MeshPhongMaterial({
                color: 0x8B4513,
                shininess: 5
            });
            const trunk = new THREE.Mesh(trunkGeometry, trunkMaterial);
            trunk.position.y = trunkHeight * 0.5;
            palmGroup.add(trunk);
            
            // Create palm leaves
            const leaves = 5 + Math.floor(Math.random() * 3);
            for (let j = 0; j < leaves; j++) {
                const leafGeometry = new THREE.ConeGeometry(0.3, 4, 4);
                const leafMaterial = new THREE.MeshPhongMaterial({
                    color: 0x2F4F4F,
                    shininess: 10
                });
                const leaf = new THREE.Mesh(leafGeometry, leafMaterial);
                
                // Position and rotate leaf
                const angle = (j * Math.PI * 2) / leaves;
                leaf.position.set(
                    Math.cos(angle) * 2,
                    trunkHeight,
                    Math.sin(angle) * 2
                );
                leaf.rotation.x = Math.PI * 0.25;
                leaf.rotation.y = angle;
                
                palmGroup.add(leaf);
            }
            
            // Position palm tree around pool
            const angle = (i * Math.PI * 2) / 6;
            const radius = 6;
            palmGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            oasisGroup.add(palmGroup);
        }
        
        // Add some vegetation around oasis
        for (let i = 0; i < 12; i++) {
            const bushGeometry = new THREE.SphereGeometry(1, 8, 6);
            const bushMaterial = new THREE.MeshPhongMaterial({
                color: 0x2E8B57,
                shininess: 5
            });
            const bush = new THREE.Mesh(bushGeometry, bushMaterial);
            
            const angle = Math.random() * Math.PI * 2;
            const radius = 8 + Math.random() * 4;
            bush.position.set(
                Math.cos(angle) * radius,
                -9,
                Math.sin(angle) * radius
            );
            
            oasisGroup.add(bush);
        }
        
        // Position oasis
        const oasisAngle = Math.random() * Math.PI * 2;
        const oasisRadius = 60;
        oasisGroup.position.set(
            Math.cos(oasisAngle) * oasisRadius,
            0,
            Math.sin(oasisAngle) * oasisRadius
        );
        
        this.scene.add(oasisGroup);
        
        // Create scattered cacti
        for (let i = 0; i < 20; i++) {
            const cactusGroup = new THREE.Group();
            
            // Main cactus body
            const height = 4 + Math.random() * 3;
            const bodyGeometry = new THREE.CylinderGeometry(
                0.8, 1,
                height,
                8
            );
            const cactusMaterial = new THREE.MeshPhongMaterial({
                color: 0x2F4F4F,
                shininess: 5
            });
            
            const body = new THREE.Mesh(bodyGeometry, cactusMaterial);
            body.position.y = height * 0.5;
            cactusGroup.add(body);
            
            // Add arms to some cacti
            if (Math.random() > 0.3) {
                const arms = Math.floor(Math.random() * 2) + 1;
                for (let j = 0; j < arms; j++) {
                    const armHeight = height * (0.3 + Math.random() * 0.3);
                    const armGeometry = new THREE.CylinderGeometry(
                        0.4, 0.4,
                        armHeight,
                        8
                    );
                    const arm = new THREE.Mesh(armGeometry, cactusMaterial);
                    
                    // Position and rotate arm
                    const armAngle = Math.random() * Math.PI * 2;
                    arm.position.set(
                        Math.cos(armAngle) * 0.8,
                        height * (0.5 + Math.random() * 0.2),
                        Math.sin(armAngle) * 0.8
                    );
                    arm.rotation.z = Math.PI * 0.25 * (Math.random() > 0.5 ? 1 : -1);
                    
                    cactusGroup.add(arm);
                }
            }
            
            // Position cactus
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 140 + 20;
            cactusGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            this.scene.add(cactusGroup);
            this.obstacles.push(cactusGroup);
        }
        
        console.log("Desert environment created with obstacles:", this.obstacles.length);
    }
    
    createGates() {
        // Create 8 gates for the drone to fly through
        for (let i = 0; i < 8; i++) {
            const gateWidth = 5;
            const gateHeight = 5;
            const gateDepth = 1;
            
            // Create gate frame
            const frameGeometry = new THREE.BoxGeometry(gateWidth, gateHeight, gateDepth);
            const frameMaterial = new THREE.MeshPhongMaterial({ 
                color: 0xff0000,
                emissive: 0xff0000,
                emissiveIntensity: 0.5
            });
            const frame = new THREE.Mesh(frameGeometry, frameMaterial);
            
            // Create gate opening (inner part)
            const openingGeometry = new THREE.BoxGeometry(gateWidth - 1, gateHeight - 1, gateDepth + 0.5);
            const openingMaterial = new THREE.MeshBasicMaterial({ 
                color: 0x000000, 
                transparent: true, 
                opacity: 0.5 
            });
            const opening = new THREE.Mesh(openingGeometry, openingMaterial);
            
            // Create gate group
            const gate = new THREE.Group();
            gate.add(frame);
            gate.add(opening);
            
            // Random position
            gate.position.set(
                (Math.random() - 0.5) * 150,
                gateHeight / 2 + 2,
                (Math.random() - 0.5) * 150
            );
            
            // Random rotation around Y axis
            gate.rotation.y = Math.random() * Math.PI * 2;
            
            gate.userData = { 
                isGate: true, 
                passed: false,
                points: 100 // Points awarded for passing through this gate
            };
            
            this.scene.add(gate);
            this.gates.push(gate);
        }
    }
    
    createBillboards() {
        // Create billboards for advertisements
        const minDistanceFromBuildings = 15; // Minimum distance from buildings
        const attempts = 10; // Maximum attempts to find valid position
        
        for (let i = 0; i < 5; i++) {
            const width = 10;
            const height = 5;
            const depth = 0.5;
            
            // Create billboard frame
            const frameGeometry = new THREE.BoxGeometry(width, height, depth);
            const frameMaterial = new THREE.MeshPhongMaterial({ color: 0x333333 });
            const frame = new THREE.Mesh(frameGeometry, frameMaterial);
            
            // Create billboard screen
            const screenGeometry = new THREE.PlaneGeometry(width - 0.5, height - 0.5);
            const screenMaterial = new THREE.MeshPhongMaterial({ 
                color: 0xffffff,
                emissive: 0xffffff,
                emissiveIntensity: 0.5
            });
            
            // Create ad texture
            const texture = this.createAdTexture(i);
            screenMaterial.map = texture;
            
            const screen = new THREE.Mesh(screenGeometry, screenMaterial);
            screen.position.z = depth / 2 + 0.01;
            
            // Create billboard group
            const billboard = new THREE.Group();
            billboard.add(frame);
            billboard.add(screen);
            
            // Find valid position
            let validPosition = false;
            let attemptCount = 0;
            
            while (!validPosition && attemptCount < attempts) {
                const x = (Math.random() - 0.5) * 180;
                const z = (Math.random() - 0.5) * 180;
                
                // Check distance from all buildings
                let tooClose = false;
                for (const obstacle of this.obstacles) {
                    if (obstacle.isBuilding) {
                        const dx = obstacle.position.x - x;
                        const dz = obstacle.position.z - z;
                        const distance = Math.sqrt(dx * dx + dz * dz);
                        
                        if (distance < minDistanceFromBuildings) {
                            tooClose = true;
                            break;
                        }
                    }
                }
                
                if (!tooClose) {
                    billboard.position.set(x, height / 2 + 10, z);
                    validPosition = true;
                }
                
                attemptCount++;
            }
            
            // If no valid position found, try a fallback position
            if (!validPosition) {
                const angle = (i / 5) * Math.PI * 2;
                const radius = 100;
                billboard.position.set(
                    Math.cos(angle) * radius,
                    height / 2 + 10,
                    Math.sin(angle) * radius
                );
            }
            
            // Random rotation around Y axis
            billboard.rotation.y = Math.random() * Math.PI * 2;
            
            billboard.castShadow = true;
            billboard.receiveShadow = true;
            
            this.scene.add(billboard);
            this.billboards.push(billboard);
        }
    }
    
    createAdTexture(index) {
        // Create a canvas for the ad texture
        const canvas = document.createElement('canvas');
        canvas.width = 512;
        canvas.height = 256;
        const context = canvas.getContext('2d');
        
        // Fill background
        context.fillStyle = '#ffffff';
        context.fillRect(0, 0, canvas.width, canvas.height);
        
        // Add border
        context.strokeStyle = '#000000';
        context.lineWidth = 10;
        context.strokeRect(5, 5, canvas.width - 10, canvas.height - 10);
        
        // Add text
        context.fillStyle = '#000000';
        context.font = 'bold 48px Arial';
        context.textAlign = 'center';
        context.textBaseline = 'middle';
        
        // Different ad content based on index
        let adText, adSubtext;
        
        switch(index % 5) {
            case 0:
                adText = 'DRONE RACING';
                adSubtext = 'Join the league today!';
                break;
            case 1:
                adText = 'SKYLINE TECH';
                adSubtext = 'Future is now';
                break;
            case 2:
                adText = 'CLOUD NINE';
                adSubtext = 'Premium drone parts';
                break;
            case 3:
                adText = 'AIR SPACE';
                adSubtext = 'Fly with style';
                break;
            case 4:
                adText = 'YOUR AD HERE';
                adSubtext = 'Contact us for sponsorship';
                break;
        }
        
        context.fillText(adText, canvas.width / 2, canvas.height / 2 - 30);
        
        context.font = '32px Arial';
        context.fillText(adSubtext, canvas.width / 2, canvas.height / 2 + 30);
        
        // Create texture from canvas
        const texture = new THREE.CanvasTexture(canvas);
        texture.needsUpdate = true;
        
        return texture;
    }
    
    createPowerups() {
        // Create powerups for the drone to collect
        const powerupCount = 10;
        
        for (let i = 0; i < powerupCount; i++) {
            const geometry = new THREE.SphereGeometry(1, 16, 16);
            const material = new THREE.MeshPhongMaterial({ 
                color: 0xffff00,
                emissive: 0xffff00,
                emissiveIntensity: 0.5,
                transparent: true,
                opacity: 0.7
            });
            
            const powerup = new THREE.Mesh(geometry, material);
            
            // Random position
            powerup.position.set(
                (Math.random() - 0.5) * 150,
                5 + Math.random() * 20,
                (Math.random() - 0.5) * 150
            );
            
            powerup.userData = { 
                isPowerup: true,
                id: i,
                collected: false,
                type: this.getRandomPowerupType(),
                points: 50 // Points awarded for collecting this powerup
            };
            
            this.scene.add(powerup);
            this.powerups.push(powerup);
        }
    }
    
    getRandomPowerupType() {
        const types = ['speed', 'shield', 'points'];
        return types[Math.floor(Math.random() * types.length)];
    }
    
    addBuildingWindows(building, width, height, depth) {
        const windowSize = 0.8;  // Slightly larger windows
        const windowSpacing = 2.5;  // Increased spacing between windows
        
        const windowsX = Math.floor(width / windowSpacing);
        const windowsY = Math.floor(height / windowSpacing);
        
        const windowGeometry = new THREE.PlaneGeometry(windowSize, windowSize);
        const windowMaterial = new THREE.MeshPhongMaterial({ 
            color: 0x88ccff,
            emissive: 0x88ccff,
            emissiveIntensity: 0.2,
            transparent: true,
            opacity: 0.9
        });
        
        // Add windows to front face with reduced density
        for (let y = 0; y < windowsY; y++) {
            for (let x = 0; x < windowsX; x++) {
                if (Math.random() > 0.5) { // 50% chance of window (reduced from 80%)
                    const window = new THREE.Mesh(windowGeometry, windowMaterial.clone());
                    window.position.set(
                        (x * windowSpacing) - (width / 2) + (windowSpacing / 2),
                        (y * windowSpacing) - (height / 2) + (windowSpacing / 2) + 1,
                        depth / 2 + 0.01
                    );
                    building.add(window);
                }
            }
        }
        
        // Add windows to back face with reduced density
        for (let y = 0; y < windowsY; y++) {
            for (let x = 0; x < windowsX; x++) {
                if (Math.random() > 0.5) { // 50% chance of window (reduced from 80%)
                    const window = new THREE.Mesh(windowGeometry, windowMaterial.clone());
                    window.position.set(
                        (x * windowSpacing) - (width / 2) + (windowSpacing / 2),
                        (y * windowSpacing) - (height / 2) + (windowSpacing / 2) + 1,
                        -depth / 2 - 0.01
                    );
                    window.rotation.y = Math.PI;
                    building.add(window);
                }
            }
        }
    }
    
    addNeonWindows(building, width, height, depth) {
        const windowSize = 0.5;
        const windowSpacing = 1.2;
        
        const windowsX = Math.floor(width / windowSpacing);
        const windowsY = Math.floor(height / windowSpacing);
        
        const neonColors = [0xff00ff, 0x00ffff, 0xff0000, 0x00ff00, 0x0000ff, 0xffff00];
        
        const windowGeometry = new THREE.PlaneGeometry(windowSize, windowSize);
        
        // Add windows to front and back faces
        for (let y = 0; y < windowsY; y++) {
            for (let x = 0; x < windowsX; x++) {
                if (Math.random() > 0.3) { // 70% chance of window
                    const color = neonColors[Math.floor(Math.random() * neonColors.length)];
                    const windowMaterial = new THREE.MeshPhongMaterial({ 
                        color: color,
                        emissive: color,
                        emissiveIntensity: 0.8,
                        transparent: true,
                        opacity: 0.9
                    });
                    
                    // Front face
                    const window1 = new THREE.Mesh(windowGeometry, windowMaterial);
                    window1.position.set(
                        (x * windowSpacing) - (width / 2) + (windowSpacing / 2),
                        (y * windowSpacing) - (height / 2) + (windowSpacing / 2) + 1,
                        depth / 2 + 0.01
                    );
                    building.add(window1);
                    
                    // Back face
                    const window2 = new THREE.Mesh(windowGeometry, windowMaterial);
                    window2.position.set(
                        (x * windowSpacing) - (width / 2) + (windowSpacing / 2),
                        (y * windowSpacing) - (height / 2) + (windowSpacing / 2) + 1,
                        -depth / 2 - 0.01
                    );
                    window2.rotation.y = Math.PI;
                    building.add(window2);
                }
            }
        }
    }

    checkGatePass(drone) {
        this.gates.forEach(gate => {
            if (!gate.userData.passed) {
                // Get gate position and drone position
                const gatePos = new THREE.Vector3();
                gate.getWorldPosition(gatePos);
                
                const dronePos = drone.mesh.position.clone();
                
                // Check if drone is close to gate
                const distance = gatePos.distanceTo(dronePos);
                
                if (distance < 5) {
                    // Check if drone is passing through gate
                    // Convert drone position to gate's local space
                    const localPos = dronePos.clone().sub(gatePos);
                    localPos.applyQuaternion(gate.quaternion.clone().inverse());
                    
                    // Check if drone is within gate bounds
                    if (Math.abs(localPos.x) < 2 && Math.abs(localPos.y) < 2 && Math.abs(localPos.z) < 2) {
                        gate.userData.passed = true;
                        console.log("Gate passed!");
                        
                        // Change gate color to green
                        gate.children[0].material.color.set(0x00ff00);
                        
                        // Add points to score
                        this.score += gate.userData.points;
                        console.log(`Score: ${this.score}`);
                        
                        // Dispatch score update event
                        const event = new CustomEvent('scoreUpdate', { detail: { score: this.score } });
                        document.dispatchEvent(event);
                    }
                }
            }
        });
        
        // Check for powerup collection
        this.powerups.forEach(powerup => {
            if (!powerup.userData.collected) {
                const powerupPos = powerup.position.clone();
                const dronePos = drone.mesh.position.clone();
                
                // Check if drone is close to powerup
                const distance = powerupPos.distanceTo(dronePos);
                
                if (distance < 2) {
                    powerup.userData.collected = true;
                    console.log(`Powerup collected: ${powerup.userData.type}`);
                    
                    // Hide the powerup
                    powerup.visible = false;
                    
                    // Add points to score
                    this.score += powerup.userData.points;
                    console.log(`Score: ${this.score}`);
                    
                    // Apply powerup effect
                    this.applyPowerupEffect(drone, powerup.userData.type);
                    
                    // Dispatch score update event
                    const event = new CustomEvent('scoreUpdate', { detail: { score: this.score } });
                    document.dispatchEvent(event);
                }
            }
        });
    }
    
    applyPowerupEffect(drone, type) {
        switch(type) {
            case 'speed':
                // Temporarily increase drone speed
                drone.speedMultiplier = 2;
                setTimeout(() => {
                    drone.speedMultiplier = 1;
                }, 5000);
                break;
            case 'shield':
                // Temporarily make drone invulnerable
                drone.hasShield = true;
                setTimeout(() => {
                    drone.hasShield = false;
                }, 5000);
                break;
            case 'points':
                // Just add extra points (already handled in collection)
                break;
        }
    }
    
    getObstacles() {
        return this.obstacles;
    }
    
    getScore() {
        return Math.floor(this.score);
    }
    
    resetScore() {
        this.score = 0;
        this.lastScoreUpdate = Date.now();
    }
    
    updateScore(deltaTime) {
        // Base score accumulation (time-based)
        const timeBonus = deltaTime * 2; // 2 points per second of flight
        
        // Altitude bonus (more points for flying higher, up to a reasonable height)
        const altitude = Math.max(0, drone.mesh.position.y + 10);
        const altitudeBonus = Math.min(altitude * 0.1, 5) * deltaTime;
        
        // Speed bonus (more points for maintaining higher speeds)
        const speed = drone.velocity.length();
        const speedBonus = Math.min(speed * 0.2, 10) * deltaTime;
        
        // Trick bonus (based on rotation)
        const rotation = Math.abs(drone.mesh.rotation.z) + Math.abs(drone.mesh.rotation.x);
        const trickBonus = rotation > Math.PI ? 5 : 0;
        
        // Update total score
        this.score += timeBonus + altitudeBonus + speedBonus + trickBonus;
    }
    
    changeEnvironment(type) {
        this.clearEnvironment();
        this.environmentType = type;
        this.createEnvironment(type);
        this.resetScore();
    }

    createClouds() {
        const createCloud = () => {
            const group = new THREE.Group();
            const geometry = new THREE.SphereGeometry(1, 16, 16);
            const material = new THREE.MeshPhongMaterial({
                color: 0xffffff,
                transparent: true,
                opacity: 0.8
            });
            
            // Create multiple spheres for one cloud
            for (let i = 0; i < 5; i++) {
                const sphere = new THREE.Mesh(geometry, material);
                sphere.position.set(
                    Math.random() * 4 - 2,
                    Math.random() * 2 - 1,
                    Math.random() * 4 - 2
                );
                sphere.scale.set(
                    Math.random() * 2 + 1,
                    Math.random() * 1 + 0.5,
                    Math.random() * 2 + 1
                );
                group.add(sphere);
            }
            
            // Random position in sky
            group.position.set(
                Math.random() * 300 - 150,
                Math.random() * 20 + 50,
                Math.random() * 300 - 150
            );
            
            return group;
        };
        
        // Create multiple clouds
        for (let i = 0; i < 20; i++) {
            const cloud = createCloud();
            this.scene.add(cloud);
        }
    }

    createBirds() {
        const createBird = () => {
            const group = new THREE.Group();
            
            // Bird body
            const bodyGeometry = new THREE.ConeGeometry(0.2, 1, 4);
            const bodyMaterial = new THREE.MeshPhongMaterial({ color: 0x333333 });
            const body = new THREE.Mesh(bodyGeometry, bodyMaterial);
            body.rotation.x = Math.PI / 2;
            group.add(body);
            
            // Wings
            const wingGeometry = new THREE.PlaneGeometry(1, 0.5);
            const wingMaterial = new THREE.MeshPhongMaterial({
                color: 0x333333,
                side: THREE.DoubleSide
            });
            
            const leftWing = new THREE.Mesh(wingGeometry, wingMaterial);
            const rightWing = new THREE.Mesh(wingGeometry, wingMaterial);
            leftWing.position.set(-0.5, 0, 0);
            rightWing.position.set(0.5, 0, 0);
            group.add(leftWing);
            group.add(rightWing);
            
            // Random position in sky
            group.position.set(
                Math.random() * 200 - 100,
                Math.random() * 20 + 30,
                Math.random() * 200 - 100
            );
            
            return group;
        };
        
        // Create multiple birds
        for (let i = 0; i < 10; i++) {
            const bird = createBird();
            this.scene.add(bird);
        }
    }

    createBonusRings() {
        const ringGeometry = new THREE.TorusGeometry(3, 0.3, 16, 32);
        const ringMaterial = new THREE.MeshPhongMaterial({
            color: 0xffff00,
            emissive: 0xffff00,
            emissiveIntensity: 0.5,
            transparent: true,
            opacity: 0.8
        });

        // Create rings in interesting positions
        for (let i = 0; i < 10; i++) {
            const ring = new THREE.Mesh(ringGeometry, ringMaterial);
            
            // Position rings between buildings
            ring.position.set(
                (Math.random() - 0.5) * 150,
                Math.random() * 30 + 10,
                (Math.random() - 0.5) * 150
            );
            
            // Random rotation
            ring.rotation.x = Math.random() * Math.PI;
            ring.rotation.y = Math.random() * Math.PI;
            
            ring.userData = {
                isRing: true,
                points: 250,
                collected: false
            };
            
            this.scene.add(ring);
            this.bonusItems = this.bonusItems || [];
            this.bonusItems.push(ring);
        }
    }

    checkCollisions(drone) {
        // Check building proximity for glow effect
        this.obstacles.forEach(obstacle => {
            if (obstacle.userData && obstacle.userData.isBuilding) {
                const distance = drone.mesh.position.distanceTo(obstacle.position);
                if (distance < 20) {
                    // Make building glow as drone approaches
                    const intensity = Math.max(0, 1 - (distance / 20)) * obstacle.userData.maxGlowIntensity;
                    obstacle.material.emissive.setHex(0x88ccff);
                    obstacle.material.emissiveIntensity = intensity;
                } else {
                    obstacle.material.emissiveIntensity = 0;
                }
            }
        });

        // Check bonus rings
        if (this.bonusItems) {
            this.bonusItems.forEach(ring => {
                if (!ring.userData.collected) {
                    const distance = drone.mesh.position.distanceTo(ring.position);
                    if (distance < 3) {
                        // Collect ring
                        ring.userData.collected = true;
                        this.score += ring.userData.points;
                        
                        // Animation effect
                        ring.material.opacity = 0.2;
                        ring.scale.multiplyScalar(2);
                        setTimeout(() => {
                            this.scene.remove(ring);
                        }, 500);

                        // Play collection sound
                        if (window.audioManager) {
                            window.audioManager.playSound('checkpoint', { volume: 0.3 });
                        }
                    }
                }
            });
        }
    }

    update(deltaTime) {
        // Update score
        this.updateScore(deltaTime);
        
        // Update other environment elements
        this.updateClouds(deltaTime);
        this.updateBirds(deltaTime);
        this.updateBonusRings(deltaTime);
        
        // Update visual effects
        this.updateVisualEffects(deltaTime);

        // Update flying objects in city environment
        this.scene.children.forEach(child => {
            if (child.userData.isCloud) {
                child.position.x += child.userData.speed * deltaTime;
                if (child.position.x > 150) child.position.x = -150;
            }
            
            if (child.userData.isBird) {
                child.position.x += child.userData.speed * deltaTime;
                if (child.position.x > 150) child.position.x = -150;
                
                // Wing flapping animation
                if (child.children.length > 1) {
                    const wingAngle = Math.sin(Date.now() * child.userData.wingSpeed) * 0.5;
                    child.children[1].rotation.z = wingAngle;
                    child.children[2].rotation.z = -wingAngle;
                }
            }
            
            if (child.userData.isBalloon) {
                child.position.x += child.userData.speed * deltaTime;
                if (child.position.x > 150) child.position.x = -150;
                
                // Gentle floating motion
                child.position.y += Math.sin(Date.now() * 0.001) * child.userData.heightVariation;
            }
            
            if (child.userData.isNeonCloud) {
                child.position.x += child.userData.speed * deltaTime;
                if (child.position.x > 150) child.position.x = -150;
                
                // Glow pulse effect
                child.children.forEach(part => {
                    part.material.emissiveIntensity = 
                        0.3 + Math.sin(Date.now() * child.userData.glowSpeed) * 0.2;
                });
            }
            
            if (child.userData.isDrone) {
                // Circular motion
                const time = Date.now() * 0.001;
                const radius = child.userData.radius || 30;
                child.position.x = Math.cos(time * child.userData.speed) * radius;
                child.position.z = Math.sin(time * child.userData.speed) * radius;
                
                // Rotation animation
                child.rotation.y += child.userData.rotationSpeed;
            }
            
            if (child.userData.isAirship) {
                child.position.x += child.userData.speed * deltaTime;
                if (child.position.x > 150) child.position.x = -150;
                
                // Glow pulse effect
                child.children.forEach(part => {
                    if (part.material.emissive) {
                        part.material.emissiveIntensity = 
                            0.3 + Math.sin(Date.now() * child.userData.glowPulse) * 0.1;
                    }
                });
            }
        });
    }

    updateClouds(deltaTime) {
        // Update cloud positions
        this.scene.children.forEach(child => {
            if (child.userData && child.userData.isCloud) {
                child.position.x += child.userData.speed * deltaTime;
                
                // If cloud moves too far right, reset to left side
                if (child.position.x > 150) {
                    child.position.x = -150;
                }
            }
        });
    }

    updateBirds(deltaTime) {
        // Update bird animations and positions
        this.scene.children.forEach(child => {
            if (child.userData && child.userData.isBird) {
                // Simple circular motion
                const time = Date.now() * 0.001;
                child.position.x = Math.sin(time) * 50;
                child.position.z = Math.cos(time) * 50;
                
                // Wing flapping animation
                if (child.children.length > 1) {
                    const wingAngle = Math.sin(time * 10) * 0.5;
                    child.children[1].rotation.z = wingAngle;
                    child.children[2].rotation.z = -wingAngle;
                }
            }
        });
    }

    updateBonusRings(deltaTime) {
        // Update bonus ring animations
        if (this.bonusItems) {
            this.bonusItems.forEach(ring => {
                if (!ring.userData.collected) {
                    ring.rotation.y += deltaTime; // Rotate uncollected rings
                    
                    // Add floating motion
                    const time = Date.now() * 0.001;
                    ring.position.y += Math.sin(time * 2) * deltaTime * 0.5;
                }
            });
        }
    }

    updateVisualEffects(deltaTime) {
        // Update environment-specific effects
        switch(this.environmentType) {
            case 'neonCity':
                this.updateNeonEffects(deltaTime);
                break;
            case 'snow':
                this.updateSnowfall(deltaTime);
                break;
            case 'desert':
                this.updateSandstorm(deltaTime);
                break;
            case 'forest':
                this.updateForestEffects(deltaTime);
                break;
        }
    }

    updateNeonEffects(deltaTime) {
        // Update neon light effects
        this.scene.children.forEach(child => {
            if (child.userData && child.userData.isNeon) {
                const time = Date.now() * 0.001;
                child.material.emissiveIntensity = 0.5 + Math.sin(time * 2) * 0.3;
            }
        });
    }

    updateSnowfall(deltaTime) {
        // Placeholder for snow particle system update
    }

    updateSandstorm(deltaTime) {
        // Placeholder for sandstorm particle system update
    }

    updateForestEffects(deltaTime) {
        // Update forest environment effects like swaying trees
        this.scene.children.forEach(child => {
            if (child.userData && child.userData.isTree) {
                const time = Date.now() * 0.001;
                child.rotation.z = Math.sin(time + child.position.x) * 0.02;
            }
        });
    }

    createForestEnvironment() {
        // Set forest atmosphere
        this.scene.background = new THREE.Color(0x90EE90); // Light green
        this.scene.fog = new THREE.FogExp2(0x90EE90, 0.008);
        
        // Create trees
        for (let i = 0; i < 100; i++) {
            const treeGroup = new THREE.Group();
            
            // Tree height and properties
            const height = Math.random() * 15 + 10;
            const trunkRadius = height * 0.05;
            const foliageRadius = height * 0.2;
            
            // Create trunk
            const trunkGeometry = new THREE.CylinderGeometry(trunkRadius, trunkRadius * 1.2, height * 0.7, 8);
            const trunkMaterial = new THREE.MeshPhongMaterial({ 
                color: 0x4A2B0F,
                roughness: 1.0
            });
            const trunk = new THREE.Mesh(trunkGeometry, trunkMaterial);
            trunk.position.y = height * 0.35;
            treeGroup.add(trunk);
            
            // Create foliage layers
            const foliageLayers = 3;
            const foliageColors = [0x1B4D3E, 0x226F54, 0x2D9067]; // Different shades of green
            
            for (let j = 0; j < foliageLayers; j++) {
                const layerGeometry = new THREE.ConeGeometry(
                    foliageRadius * (1 - j * 0.2),
                    height * 0.5,
                    8
                );
                const layerMaterial = new THREE.MeshPhongMaterial({
                    color: foliageColors[j % foliageColors.length],
                    shininess: 0
                });
                const layer = new THREE.Mesh(layerGeometry, layerMaterial);
                layer.position.y = height * (0.5 + j * 0.2);
                treeGroup.add(layer);
            }
            
            // Random position within bounds
            const angle = Math.random() * Math.PI * 2;
            const positionRadius = Math.random() * 140 + 10;
            treeGroup.position.set(
                Math.cos(angle) * positionRadius,
                -10,
                Math.sin(angle) * positionRadius
            );
            
            // Random rotation
            treeGroup.rotation.y = Math.random() * Math.PI * 2;
            
            // Add to scene and obstacles
            this.scene.add(treeGroup);
            this.obstacles.push(treeGroup);
        }
        
        // Create forest floor vegetation
        for (let i = 0; i < 200; i++) {
            const bushGroup = new THREE.Group();
            
            // Bush properties
            const size = Math.random() * 2 + 1;
            
            // Create bush geometry
            const bushGeometry = new THREE.SphereGeometry(size, 8, 6);
            const bushMaterial = new THREE.MeshPhongMaterial({
                color: Math.random() > 0.5 ? 0x2D5A27 : 0x3A7D44,
                shininess: 0
            });
            const bush = new THREE.Mesh(bushGeometry, bushMaterial);
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 150;
            bush.position.set(
                Math.cos(angle) * radius,
                -9 + Math.random(),
                Math.sin(angle) * radius
            );
            
            bushGroup.add(bush);
            this.scene.add(bushGroup);
        }
        
        // Create rocks
        for (let i = 0; i < 30; i++) {
            const rockGroup = new THREE.Group();
            
            // Rock properties
            const size = Math.random() * 3 + 1;
            
            // Create rock geometry
            const rockGeometry = new THREE.DodecahedronGeometry(size, 0);
            const rockMaterial = new THREE.MeshPhongMaterial({
                color: 0x808080,
                roughness: 1.0
            });
            const rock = new THREE.Mesh(rockGeometry, rockMaterial);
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 150;
            rock.position.set(
                Math.cos(angle) * radius,
                -9,
                Math.sin(angle) * radius
            );
            
            // Random rotation
            rock.rotation.set(
                Math.random() * Math.PI,
                Math.random() * Math.PI,
                Math.random() * Math.PI
            );
            
            rockGroup.add(rock);
            this.scene.add(rockGroup);
            this.obstacles.push(rockGroup);
        }
        
        // Create forest ground
        const groundGeometry = new THREE.PlaneGeometry(300, 300);
        const groundMaterial = new THREE.MeshPhongMaterial({
            color: 0x355E3B, // Forest green
            roughness: 1.0
        });
        const forestGround = new THREE.Mesh(groundGeometry, groundMaterial);
        forestGround.rotation.x = -Math.PI / 2;
        forestGround.position.y = -10;
        this.scene.add(forestGround);
    }

    createMountainEnvironment() {
        // Set mountain atmosphere
        this.scene.background = new THREE.Color(0xADD8E6); // Light blue sky
        this.scene.fog = new THREE.FogExp2(0xADD8E6, 0.003);
        
        // Create mountain ranges
        for (let i = 0; i < 8; i++) {
            const mountainGroup = new THREE.Group();
            
            // Create main mountain peak
            const height = 30 + Math.random() * 40;
            const baseWidth = height * 0.8;
            const mountainGeometry = new THREE.ConeGeometry(baseWidth, height, 8);
            const mountainMaterial = new THREE.MeshPhongMaterial({
                color: 0x808080, // Gray base color
                roughness: 1.0
            });
            
            const mountain = new THREE.Mesh(mountainGeometry, mountainMaterial);
            
            // Add snow cap
            const snowCapHeight = height * 0.3;
            const snowCapGeometry = new THREE.ConeGeometry(baseWidth * 0.4, snowCapHeight, 8);
            const snowMaterial = new THREE.MeshPhongMaterial({
                color: 0xFFFFFF,
                shininess: 70
            });
            const snowCap = new THREE.Mesh(snowCapGeometry, snowMaterial);
            snowCap.position.y = height * 0.4;
            
            mountainGroup.add(mountain);
            mountainGroup.add(snowCap);
            
            // Add rocky details
            for (let j = 0; j < 5; j++) {
                const rockSize = Math.random() * 5 + 2;
                const rockGeometry = new THREE.DodecahedronGeometry(rockSize, 1);
                const rockMaterial = new THREE.MeshPhongMaterial({
                    color: 0x696969,
                    roughness: 0.8
                });
                const rock = new THREE.Mesh(rockGeometry, rockMaterial);
                
                // Position rocks around mountain base
                const angle = Math.random() * Math.PI * 2;
                const distance = baseWidth * 0.7;
                rock.position.set(
                    Math.cos(angle) * distance,
                    rockSize,
                    Math.sin(angle) * distance
                );
                
                mountainGroup.add(rock);
            }
            
            // Position mountain group
            const angle = (i * Math.PI * 2) / 8 + Math.random() * 0.5;
            const radius = 80 + Math.random() * 40;
            mountainGroup.position.set(
                Math.cos(angle) * radius,
                -10,
                Math.sin(angle) * radius
            );
            
            // Random rotation
            mountainGroup.rotation.y = Math.random() * Math.PI * 2;
            
            this.scene.add(mountainGroup);
            this.obstacles.push(mountainGroup);
        }
        
        // Add scattered rocks and boulders
        for (let i = 0; i < 40; i++) {
            const rockGroup = new THREE.Group();
            
            const size = Math.random() * 4 + 2;
            const rockGeometry = new THREE.DodecahedronGeometry(size, 1);
            const rockMaterial = new THREE.MeshPhongMaterial({
                color: 0x808080,
                roughness: 1.0
            });
            const rock = new THREE.Mesh(rockGeometry, rockMaterial);
            
            // Random position
            const angle = Math.random() * Math.PI * 2;
            const radius = Math.random() * 140 + 20;
            rock.position.set(
                Math.cos(angle) * radius,
                -8,
                Math.sin(angle) * radius
            );
            
            // Random rotation
            rock.rotation.set(
                Math.random() * Math.PI,
                Math.random() * Math.PI,
                Math.random() * Math.PI
            );
            
            rockGroup.add(rock);
            this.scene.add(rockGroup);
            this.obstacles.push(rockGroup);
        }
        
        // Create mountain ground
        const groundGeometry = new THREE.PlaneGeometry(300, 300);
        const groundMaterial = new THREE.MeshPhongMaterial({
            color: 0x8B4513, // Earthy brown
            roughness: 1.0
        });
        const mountainGround = new THREE.Mesh(groundGeometry, groundMaterial);
        mountainGround.rotation.x = -Math.PI / 2;
        mountainGround.position.y = -10;
        mountainGround.isGround = true;
        this.scene.add(mountainGround);
    }
} 