class Mission {
    constructor(scene, environment) {
        this.scene = scene;
        this.environment = environment;
        this.missionType = null;
        this.score = 0;
        this.startTime = 0;
        this.checkpoints = [];
        this.deliveryPoints = [];
        this.stuntPoints = 0;
        this.lastStuntTime = 0;
        this.combo = 1;
        this.isActive = false;
        this.objectives = [];
        this.instructionElement = null;
        this.guideArrows = [];
        this.audioManager = new AudioManager();
        this.createInstructionElement();
    }

    createInstructionElement() {
        // Create instruction element if it doesn't exist
        if (!document.getElementById('missionInstructions')) {
            this.instructionElement = document.createElement('div');
            this.instructionElement.id = 'missionInstructions';
            this.instructionElement.style.position = 'absolute';
            this.instructionElement.style.top = '100px';
            this.instructionElement.style.right = '20px';
            this.instructionElement.style.padding = '15px';
            this.instructionElement.style.backgroundColor = 'rgba(0, 0, 0, 0.7)';
            this.instructionElement.style.color = 'white';
            this.instructionElement.style.borderRadius = '5px';
            this.instructionElement.style.fontFamily = 'Arial, sans-serif';
            this.instructionElement.style.zIndex = '1000';
            this.instructionElement.style.minWidth = '250px';
            this.instructionElement.style.display = 'none'; // Initially hide instructions
            document.body.appendChild(this.instructionElement);
        }
    }

    createGuideArrow(position, target) {
        const arrowGeometry = new THREE.ConeGeometry(1.5, 4, 8);
        const arrowMaterial = new THREE.MeshPhongMaterial({
            color: 0xffff00,
            transparent: true,
            opacity: 0.8,
            emissive: 0xffff00,
            emissiveIntensity: 0.5
        });
        const arrow = new THREE.Mesh(arrowGeometry, arrowMaterial);
        arrow.position.copy(position);
        arrow.userData.target = target;
        arrow.userData.initialY = position.y;
        arrow.userData.animationOffset = Math.random() * Math.PI * 2; // Random phase offset for animation
        this.scene.add(arrow);
        this.guideArrows.push(arrow);
        return arrow;
    }

    updateGuideArrows(dronePosition) {
        const now = performance.now() / 1000; // Current time in seconds
        
        this.guideArrows.forEach(arrow => {
            const target = arrow.userData.target;
            if (target) {
                // Point arrow from drone to target
                const direction = new THREE.Vector3()
                    .subVectors(target.position, dronePosition)
                    .normalize();
                
                // Position arrow above drone
                arrow.position.copy(dronePosition)
                    .add(new THREE.Vector3(0, 5, 0)) // Raise arrow above drone
                    .add(direction.multiplyScalar(8));
                
                // Add hovering animation
                const hoverHeight = Math.sin(now * 2 + arrow.userData.animationOffset) * 0.5;
                arrow.position.y += hoverHeight;
                
                // Make arrow point to target
                arrow.lookAt(target.position);
                
                // Rotate arrow to point downward
                arrow.rotateX(Math.PI / 2);
                
                // Pulse opacity
                arrow.material.opacity = 0.5 + Math.sin(now * 3 + arrow.userData.animationOffset) * 0.3;
                
                // Update emissive intensity for glow effect
                arrow.material.emissiveIntensity = 0.3 + Math.sin(now * 2 + arrow.userData.animationOffset) * 0.2;
                
                // Hide arrow if target is passed/delivered
                if (target.userData.passed || target.userData.delivered) {
                    arrow.visible = false;
                } else {
                    arrow.visible = true;
                }
            }
        });
    }

    startMission(type) {
        this.missionType = type || 'none';  // Default to 'none' if no type provided
        this.score = 0;
        this.startTime = performance.now();
        this.isActive = true;
        this.checkpoints = [];
        this.deliveryPoints = [];
        this.stuntPoints = 0;
        this.combo = 1;
        this.objectives = [];

        // Clear existing mission objects
        this.clearMissionObjects();

        switch(this.missionType) {
            case 'race':
                this.setupRaceMission();
                break;
            case 'stunt':
                this.setupStuntMission();
                break;
            case 'delivery':
                this.setupDeliveryMission();
                break;
            case 'none':
            default:
                this.setupFreeFlight();
                break;
        }

        // Show instructions when mission starts
        if (this.instructionElement) {
            this.instructionElement.style.display = 'block';
        }

        // Update instructions based on mission type
        this.updateInstructions(this.missionType);
        
        // Update score display immediately
        this.updateScoreDisplay();
    }

    setupFreeFlight() {
        // Clear any existing objectives
        this.objectives = [];
        this.score = 0;
        this.isActive = true;
        
        // Show instruction element for free flight mode
        if (this.instructionElement) {
            this.instructionElement.style.display = 'block';
        }
    }

    setupRaceMission() {
        const checkpointCount = 10;
        const radius = 80;
        const minHeight = 10;
        const maxHeight = 40;
        const minDistanceFromObstacles = 15;

        for (let i = 0; i < checkpointCount; i++) {
            const angle = (i / checkpointCount) * Math.PI * 2;
            let validPosition = false;
            let attempts = 0;
            
            while (!validPosition && attempts < 10) {
                const x = Math.cos(angle) * (radius + (Math.random() - 0.5) * 40);
                const y = minHeight + Math.random() * (maxHeight - minHeight);
                const z = Math.sin(angle) * (radius + (Math.random() - 0.5) * 40);
                
                // Check distance from obstacles
                let tooClose = false;
                for (const obstacle of this.environment.obstacles) {
                    const dx = obstacle.position.x - x;
                    const dy = obstacle.position.y - y;
                    const dz = obstacle.position.z - z;
                    const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);
                    
                    if (distance < minDistanceFromObstacles) {
                        tooClose = true;
                        break;
                    }
                }
                
                if (!tooClose) {
                    this.createCheckpoint(x, y, z, i);
                    validPosition = true;
                }
                
                attempts++;
            }
            
            // Fallback position if no valid position found
            if (!validPosition) {
                this.createCheckpoint(
                    Math.cos(angle) * radius,
                    30,
                    Math.sin(angle) * radius,
                    i
                );
            }
        }
    }

    setupStuntMission() {
        // Create stunt rings at various angles
        const ringCount = 15;
        const radius = 60;
        
        for (let i = 0; i < ringCount; i++) {
            const angle = (i / ringCount) * Math.PI * 2;
            const x = Math.cos(angle) * radius;
            const y = 20 + Math.sin(i * 1.5) * 15;
            const z = Math.sin(angle) * radius;
            
            this.createStuntRing(x, y, z);
        }
    }

    setupDeliveryMission() {
        // Create delivery points
        const pointCount = 5;
        const radius = 70;
        
        for (let i = 0; i < pointCount; i++) {
            const angle = (i / pointCount) * Math.PI * 2;
            const x = Math.cos(angle) * radius;
            const z = Math.sin(angle) * radius;
            
            this.createDeliveryPoint(x, z);
        }
    }

    createCheckpoint(x, y, z, index) {
        const geometry = new THREE.TorusGeometry(5, 0.5, 16, 32);
        const material = new THREE.MeshPhongMaterial({
            color: 0x00ff00,
            emissive: 0x00ff00,
            emissiveIntensity: 0.5,
            transparent: true,
            opacity: 0.8
        });
        
        const checkpoint = new THREE.Mesh(geometry, material);
        checkpoint.position.set(x, y, z);
        checkpoint.rotation.x = Math.PI / 2;
        
        checkpoint.userData = {
            isCheckpoint: true,
            index: index,
            passed: false
        };
        
        this.scene.add(checkpoint);
        this.checkpoints.push(checkpoint);
    }

    createStuntRing(x, y, z) {
        const geometry = new THREE.TorusGeometry(4, 0.3, 16, 32);
        const material = new THREE.MeshPhongMaterial({
            color: 0xff00ff,
            emissive: 0xff00ff,
            emissiveIntensity: 0.5,
            transparent: true,
            opacity: 0.8
        });
        
        const ring = new THREE.Mesh(geometry, material);
        ring.position.set(x, y, z);
        ring.rotation.x = Math.PI / 2;
        
        ring.userData = {
            isStuntRing: true,
            passed: false,
            points: 100
        };
        
        this.scene.add(ring);
        this.checkpoints.push(ring);
    }

    createDeliveryPoint(x, z) {
        const geometry = new THREE.BoxGeometry(4, 4, 4);
        const material = new THREE.MeshPhongMaterial({
            color: 0x00ffff,
            emissive: 0x00ffff,
            emissiveIntensity: 0.5,
            transparent: true,
            opacity: 0.8
        });
        
        const point = new THREE.Mesh(geometry, material);
        point.position.set(x, 2, z);
        
        point.userData = {
            isDeliveryPoint: true,
            delivered: false,
            points: 200
        };
        
        this.scene.add(point);
        this.deliveryPoints.push(point);
    }

    clearMissionObjects() {
        // Remove checkpoints
        for (const checkpoint of this.checkpoints) {
            this.scene.remove(checkpoint);
        }
        this.checkpoints = [];
        
        // Remove delivery points
        for (const point of this.deliveryPoints) {
            this.scene.remove(point);
        }
        this.deliveryPoints = [];

        // Remove guide arrows
        for (const arrow of this.guideArrows) {
            this.scene.remove(arrow);
        }
        this.guideArrows = [];
    }

    checkProgress(drone) {
        if (!this.isActive) return;

        // Update instructions for all mission types
        this.updateInstructions(this.missionType);
        
        // Only update guide arrows for missions with targets
        if (this.missionType !== 'none') {
            this.updateGuideArrows(drone.mesh.position);
        }

        switch(this.missionType) {
            case 'race':
                this.checkRaceProgress(drone);
                break;
            case 'stunt':
                this.checkStuntProgress(drone);
                break;
            case 'delivery':
                this.checkDeliveryProgress(drone);
                break;
            default:
                this.checkFreeFlightProgress(drone);
        }
        
        // Update score display after any changes
        this.updateScoreDisplay();
    }

    checkRaceProgress(drone) {
        // Check each checkpoint
        for (const checkpoint of this.checkpoints) {
            if (!checkpoint.userData.passed) {
                const distance = drone.mesh.position.distanceTo(checkpoint.position);
                if (distance < 5) {
                    checkpoint.userData.passed = true;
                    checkpoint.material.color.setHex(0xff0000);
                    checkpoint.material.emissive.setHex(0xff0000);
                    
                    // Award points based on speed
                    const speedBonus = Math.floor(drone.velocity.length() * 10);
                    this.score += 100 + speedBonus;
                    
                    // Check if race completed
                    if (this.checkpoints.every(cp => cp.userData.passed)) {
                        const timeBonus = Math.max(0, 30000 - (performance.now() - this.startTime));
                        this.score += Math.floor(timeBonus / 100);
                        this.isActive = false;
                    }
                }
            }
        }
    }

    checkStuntProgress(drone) {
        // Check stunt rings
        for (const ring of this.checkpoints) {
            if (!ring.userData.passed) {
                const distance = drone.mesh.position.distanceTo(ring.position);
                if (distance < 4) {
                    ring.userData.passed = true;
                    ring.material.color.setHex(0xff0000);
                    ring.material.emissive.setHex(0xff0000);
                    
                    // Award points with combo multiplier
                    this.score += ring.userData.points * this.combo;
                    this.combo++;
                }
            }
        }
        
        // Check for stunts
        if (drone.isDoingStunt) {
            const now = performance.now();
            if (now - this.lastStuntTime > 1000) { // Minimum 1 second between stunt scores
                this.stuntPoints += 50 * this.combo;
                this.lastStuntTime = now;
            }
        } else {
            // Reset combo if not doing stunts for 3 seconds
            if (performance.now() - this.lastStuntTime > 3000) {
                this.combo = 1;
            }
        }
    }

    checkDeliveryProgress(drone) {
        // Check delivery points
        for (const point of this.deliveryPoints) {
            if (!point.userData.delivered) {
                const distance = drone.mesh.position.distanceTo(point.position);
                if (distance < 5 && drone.velocity.length() < 5) { // Must be slow for delivery
                    point.userData.delivered = true;
                    point.material.color.setHex(0xff0000);
                    point.material.emissive.setHex(0xff0000);
                    
                    // Award points based on drone health
                    const healthBonus = Math.floor(drone.health * 100);
                    this.score += point.userData.points + healthBonus;
                    
                    // Check if all deliveries completed
                    if (this.deliveryPoints.every(p => p.userData.delivered)) {
                        const timeBonus = Math.max(0, 60000 - (performance.now() - this.startTime));
                        this.score += Math.floor(timeBonus / 100);
                        this.isActive = false;
                    }
                }
            }
        }
    }

    checkFreeFlightProgress(drone) {
        // Check objectives
        for (const objective of this.objectives) {
            if (!objective.completed) {
                switch(objective.type) {
                    case 'altitude':
                        if (drone.mesh.position.y > objective.target) {
                            objective.completed = true;
                            this.score += objective.points;
                        }
                        break;
                    case 'speed':
                        if (drone.velocity.length() > objective.target) {
                            objective.completed = true;
                            this.score += objective.points;
                        }
                        break;
                    case 'time':
                        if ((performance.now() - this.startTime) / 1000 > objective.target) {
                            objective.completed = true;
                            this.score += objective.points;
                        }
                        break;
                }
            }
        }
        
        // Award points for smooth flying
        if (drone.velocity.length() > 10 && drone.health > 0.8) {
            this.score += Math.floor(drone.velocity.length() / 10);
        }
    }

    getScore() {
        return this.score;
    }

    getMissionStatus() {
        const elapsedTime = (performance.now() - this.startTime) / 1000;
        
        return {
            type: this.missionType,
            score: this.score,
            time: elapsedTime,
            isActive: this.isActive,
            checkpointsPassed: this.checkpoints.filter(cp => cp.userData.passed).length,
            totalCheckpoints: this.checkpoints.length,
            combo: this.combo,
            stuntPoints: this.stuntPoints,
            deliveriesCompleted: this.deliveryPoints.filter(p => p.userData.delivered).length,
            totalDeliveries: this.deliveryPoints.length,
            objectives: this.objectives.map(obj => ({
                type: obj.type,
                completed: obj.completed,
                target: obj.target
            }))
        };
    }

    updateInstructions(type) {
        if (!this.instructionElement) {
            return;
        }

        let instructions = '';
        switch(type) {
            case 'none':
                instructions = `
                    <h3>Free Flight Mode</h3>
                    <p>🎮 Practice your flying skills freely!</p>
                    <p>🎯 Explore the environment</p>
                    <p>💫 Try different maneuvers</p>
                    <p>⚡ Use boost with 'B' key</p>
                `;
                break;
            case 'race':
                instructions = `
                    <h3>Race Mission</h3>
                    <p>🏁 Fly through all checkpoints as fast as possible!</p>
                    <p>💨 Speed bonus for fast completion</p>
                `;
                break;
            case 'stunt':
                instructions = `
                    <h3>Stunt Mission</h3>
                    <p>🎯 Fly through stunt rings</p>
                    <p>🔄 Perform stunts between rings</p>
                    <p>🎮 Chain moves for combo multiplier</p>
                `;
                break;
            case 'delivery':
                instructions = `
                    <h3>Delivery Mission</h3>
                    <p>📦 Deliver packages to marked locations</p>
                    <p>⚠️ Land carefully to deliver</p>
                    <p>❤️ Health bonus for careful delivery</p>
                `;
                break;
        }
        
        if (this.instructionElement) {
            this.instructionElement.innerHTML = instructions;
            this.instructionElement.style.display = 'block';
        }
    }

    updateScoreDisplay() {
        // Update score in the mission instructions
        if (this.instructionElement) {
            const scoreElement = this.instructionElement.querySelector('.mission-score');
            if (scoreElement) {
                scoreElement.textContent = this.score;
            }
        }
    }

    updateScore(points) {
        this.score += points;
        const scoreElement = document.getElementById('score');
        if (scoreElement) {
            scoreElement.textContent = `Score: ${this.score}`;
        }
    }

    checkObjectives() {
        if (!this.objectives || !this.drone) return;

        this.objectives.forEach((objective, index) => {
            if (!objective.completed && this.isNearPosition(this.drone.mesh.position, objective.position, 5)) {
                objective.completed = true;
                this.updateScore(100); // Award points for completing objective
                
                // Update objective marker visibility
                if (objective.marker) {
                    objective.marker.visible = false;
                }
                
                // Trigger completion effects
                this.onObjectiveComplete(index);
            }
        });

        // Check if all objectives are completed
        if (this.objectives.every(obj => obj.completed)) {
            this.missionComplete();
        }
    }

    missionComplete() {
        if (!this.isCompleted) {
            this.isCompleted = true;
            this.updateScore(500); // Bonus points for completing mission
            
            // Show mission complete message
            const statusElement = document.getElementById('missionStatus');
            if (statusElement) {
                statusElement.textContent = 'Mission Complete!';
                statusElement.style.backgroundColor = 'rgba(0, 255, 0, 0.2)';
            }
        }
    }
} 