class Physics {
    static checkCollision(drone, obstacles) {
        const droneBox = new THREE.Box3().setFromObject(drone.mesh);
        let collided = false;
        
        for (const obstacle of obstacles) {
            // Skip gates in regular collision check
            if (obstacle.isGate) continue;

            const obstacleBox = new THREE.Box3().setFromObject(obstacle);
            if (droneBox.intersectsBox(obstacleBox)) {
                collided = true;
                
                // Calculate collision normal and penetration depth
                const droneCenter = new THREE.Vector3();
                const obstacleCenter = new THREE.Vector3();
                droneBox.getCenter(droneCenter);
                obstacleBox.getCenter(obstacleCenter);
                
                const normal = droneCenter.sub(obstacleCenter).normalize();
                
                // Bounce effect with energy loss
                const bounceFactor = 0.5;
                const currentSpeed = drone.velocity.length();
                
                // Project velocity onto collision normal
                const dotProduct = drone.velocity.dot(normal);
                const reflection = normal.multiplyScalar(dotProduct * 2);
                drone.velocity.sub(reflection).multiplyScalar(bounceFactor);
                
                // Add some random rotation on impact
                const impactForce = Math.min(currentSpeed * 0.2, 0.5);
                drone.mesh.rotation.x += (Math.random() - 0.5) * impactForce;
                drone.mesh.rotation.z += (Math.random() - 0.5) * impactForce;
                
                // Trigger crash effect only if impact is significant
                if (currentSpeed > 5) {
                    drone.crash();
                    // Ensure damage is applied based on impact speed
                    const damageMultiplier = Math.min(currentSpeed / 10, 2); // Cap at 2x damage
                    drone.takeDamage(drone.damagePerCrash * damageMultiplier);
                }
                break;
            }
        }

        // Ground collision/landing check with improved physics
        if (drone.mesh.position.y < -9) {
            drone.mesh.position.y = -9;
            
            // Check if this is a landing or a crash
            const speed = drone.velocity.length();
            const verticalSpeed = Math.abs(drone.velocity.y);
            const tiltAngle = Math.max(
                Math.abs(drone.mesh.rotation.x),
                Math.abs(drone.mesh.rotation.z)
            );

            // More forgiving landing conditions
            if (verticalSpeed < 4 && speed < 6 && tiltAngle < 0.4) {
                // Safe landing with gentle settling
                drone.land();
                drone.velocity.multiplyScalar(0.8);
                drone.velocity.y = 0;
                
                // Gradually level out the drone
                drone.mesh.rotation.x *= 0.8;
                drone.mesh.rotation.z *= 0.8;
            } else {
                // Crash landing with bounce and damage
                const bounceEnergy = Math.min(verticalSpeed * 0.3, 0.6);
                drone.velocity.y = Math.abs(drone.velocity.y) * bounceEnergy;
                drone.velocity.multiplyScalar(0.7); // Reduce horizontal speed
                
                // Add some rotation based on impact
                const impactRotation = Math.min(speed * 0.1, 0.3);
                drone.mesh.rotation.x += (Math.random() - 0.5) * impactRotation;
                drone.mesh.rotation.z += (Math.random() - 0.5) * impactRotation;
                
                // Apply crash damage based on impact speed
                const damageMultiplier = Math.min(speed / 8, 2); // Cap at 2x damage
                drone.takeDamage(drone.damagePerCrash * damageMultiplier);
                drone.crash();
            }
        }

        return collided;
    }

    static applyBoundaries(drone, bounds) {
        const margin = 2; // Boundary margin
        const bounceFactor = 0.6; // Energy retention on boundary collision
        
        // X-axis boundaries
        if (Math.abs(drone.mesh.position.x) > bounds.x - margin) {
            const direction = drone.mesh.position.x > 0 ? 1 : -1;
            drone.mesh.position.x = (bounds.x - margin) * direction;
            drone.velocity.x *= -bounceFactor;
        }
        
        // Y-axis boundaries (height)
        if (drone.mesh.position.y > bounds.y - margin) {
            drone.mesh.position.y = bounds.y - margin;
            drone.velocity.y *= -bounceFactor;
        }
        
        // Z-axis boundaries
        if (Math.abs(drone.mesh.position.z) > bounds.z - margin) {
            const direction = drone.mesh.position.z > 0 ? 1 : -1;
            drone.mesh.position.z = (bounds.z - margin) * direction;
            drone.velocity.z *= -bounceFactor;
        }
    }
} 