let scene, camera, renderer, drone, controls, environment, mission;
let obstacles = [];
let lastTime = performance.now();
let frameCount = 0;
let lastFpsUpdate = 0;
let currentFps = 0;
let selectedDroneType = null;
let selectedEnvironment = null;
let selectedMission = null;
let audioManager;
let hudPanel;
let gameStartTime = 0;

function init() {
    console.log("Starting initialization...");

    try {
        // Create scene
        scene = new THREE.Scene();
        scene.background = new THREE.Color(0x87CEEB);  // Sky blue
        console.log("Scene created");

        // Create camera
        camera = new THREE.PerspectiveCamera(75, window.innerWidth / window.innerHeight, 0.1, 1000);
        camera.position.set(0, 15, 20);
        camera.lookAt(0, 0, 0);
        console.log("Camera created");

        // Create renderer with optimized settings
        renderer = new THREE.WebGLRenderer({ 
            antialias: true,
            alpha: true,
            powerPreference: "high-performance",
            precision: "highp",
            stencil: false
        });
        renderer.setSize(window.innerWidth, window.innerHeight);
        renderer.setPixelRatio(window.devicePixelRatio);
        renderer.shadowMap.enabled = true;
        renderer.shadowMap.type = THREE.PCFSoftShadowMap;
        document.getElementById('container').appendChild(renderer.domElement);
        console.log("Renderer created");

        // Initialize audio manager
        audioManager = new AudioManager();
        console.log("Audio manager created");

        // Add lighting
        const ambientLight = new THREE.AmbientLight(0xffffff, 1);
        scene.add(ambientLight);

        const directionalLight = new THREE.DirectionalLight(0xffffff, 0.8);
        directionalLight.position.set(50, 200, 100);
        directionalLight.castShadow = true;
        scene.add(directionalLight);
        console.log("Lights added");

        // Create environment
        environment = new Environment(scene);
        console.log("Environment created");

        // Create mission system
        mission = new Mission(scene, environment);
        console.log("Mission system created");

        // Initialize minimap
        initializeMinimap();

        // Setup UI event listeners
        setupUIEventListeners();
        console.log("UI event listeners set up");

        // Show menu and ensure first section is visible
        const menuElement = document.getElementById('menu');
        if (menuElement) {
            menuElement.classList.remove('hidden');
            menuElement.style.display = 'block';
            
            // Hide all sections except the first one
            document.querySelectorAll('.menu-section').forEach((section, index) => {
                if (index === 0) {
                    section.classList.remove('hidden');
                } else {
                    section.classList.add('hidden');
                }
            });
        }

        // Handle window resize
        window.addEventListener('resize', onWindowResize, false);

        // Add event listener for key presses
        window.addEventListener('keydown', onKeyDown, false);

        console.log("Initialization complete");
        
        // Start animation loop
        animate();

        // Create HUD panel
        hudPanel = new HUDPanel();
    } catch (error) {
        console.error("Error during initialization:", error);
        console.error(error.stack);
    }
}

function createDrone() {
    // Remove existing drone if any
    if (drone && drone.mesh) {
        scene.remove(drone.mesh);
    }

    // Create new drone with selected type and required parameters
    const droneType = selectedDroneType || 'racing';
    console.log('Creating drone of type:', droneType);
    
    drone = new Drone(droneType, scene, audioManager);
    
    // Create new controls for the drone
    controls = new Controls(drone);

    // Reset camera to follow new drone
    camera.position.set(0, 15, 20);
    camera.lookAt(0, 0, 0);

    return drone;
}

function selectDrone(type) {
    console.log('Selecting drone:', type);
    selectedDroneType = type;
    
    // Update visual selection
    document.querySelectorAll('.drone-option').forEach(option => {
        option.classList.remove('selected');
        if (option.dataset.drone === type) {
            option.classList.add('selected');
            option.style.borderColor = '#88ccff';
            option.style.boxShadow = '0 0 20px rgba(136, 204, 255, 0.5)';
        } else {
            option.style.borderColor = 'rgba(136, 204, 255, 0.3)';
            option.style.boxShadow = 'none';
        }
    });
    
    // Enable next button
    const nextBtn = document.querySelector('#droneSelection .next-btn');
    if (nextBtn) {
        nextBtn.disabled = false;
    }
}

function selectEnvironment(type) {
    console.log('Selecting environment:', type);
    selectedEnvironment = type;
    
    // Update visual selection
    document.querySelectorAll('.environment-option').forEach(option => {
        option.classList.remove('selected');
        if (option.dataset.env === type) {
            option.classList.add('selected');
            option.style.borderColor = '#88ccff';
            option.style.boxShadow = '0 0 20px rgba(136, 204, 255, 0.5)';
        } else {
            option.style.borderColor = 'rgba(136, 204, 255, 0.3)';
            option.style.boxShadow = 'none';
        }
    });
    
    // Enable next button
    const nextBtn = document.querySelector('#environmentSelection .next-btn');
    if (nextBtn) {
        nextBtn.disabled = false;
    }
}

function selectMission(type) {
    console.log('Selecting mission:', type);
    selectedMission = type;
    
    // Update visual selection
    document.querySelectorAll('.mission-option').forEach(option => {
        option.classList.remove('selected');
        if (option.dataset.mission === type) {
            option.classList.add('selected');
            option.style.borderColor = '#88ccff';
            option.style.boxShadow = '0 0 20px rgba(136, 204, 255, 0.5)';
        } else {
            option.style.borderColor = 'rgba(136, 204, 255, 0.3)';
            option.style.boxShadow = 'none';
        }
    });
}

function nextStep(step) {
    // Hide current section
    document.querySelectorAll('.menu-section').forEach(section => {
        section.classList.add('hidden');
    });
    
    // Show next section
    const nextSection = document.getElementById(`${step}Selection`);
    if (nextSection) {
        nextSection.classList.remove('hidden');
    }
    
    // Update progress indicator
    updateProgress(step);
}

function previousStep(step) {
    // Hide current section
    document.querySelectorAll('.menu-section').forEach(section => {
        section.classList.add('hidden');
    });
    
    // Show previous section
    const prevSection = document.getElementById(`${step}Selection`);
    if (prevSection) {
        prevSection.classList.remove('hidden');
    }
    
    // Update progress indicator
    updateProgress(step);
}

function updateProgress(currentStep) {
    const steps = ['drone', 'environment', 'mission'];
    const currentIndex = steps.indexOf(currentStep);
    
    document.querySelectorAll('.progress-indicator .step').forEach((step, index) => {
        step.classList.remove('active', 'completed');
        if (index < currentIndex) {
            step.classList.add('completed');
        } else if (index === currentIndex) {
            step.classList.add('active');
        }
    });
}

function onWindowResize() {
    camera.aspect = window.innerWidth / window.innerHeight;
    camera.updateProjectionMatrix();
    renderer.setSize(window.innerWidth, window.innerHeight);
}

function onKeyDown(event) {
    console.log(`Key pressed: ${event.key}`); // Debug log
    if (event.key === 'r' || event.key === 'R') {
        if (drone) {
            drone.reset();
            console.log("Drone has been reset to the original position.");
        }
    } else if (event.key === 'm' || event.key === 'M') {
        // Toggle menu visibility
        const menuElement = document.getElementById('menu');
        menuElement.classList.toggle('hidden');
        menuElement.style.display = menuElement.classList.contains('hidden') ? 'none' : 'block';
    }
}

function updateHUD() {
    try {
        const now = performance.now();
        frameCount++;
        
        if (now - lastFpsUpdate >= 1000) {
            currentFps = Math.round((frameCount * 1000) / (now - lastFpsUpdate));
            frameCount = 0;
            lastFpsUpdate = now;
        }

        if (drone && drone.mesh) {
            // Get all necessary stats
            const droneStats = drone.getFlightStats();
            const altitude = Math.max(0, (drone.mesh.position.y + 10).toFixed(1));
            const speed = drone.velocity.length().toFixed(1);
            const thrustPercentage = ((drone.thrust / 30) * 100).toFixed(0);
            const flightTime = (now - gameStartTime) / 1000; // Calculate flight time in seconds

            // Only count score when drone is moving significantly
            const minSpeedThreshold = 5;
            if (speed < minSpeedThreshold && selectedMission === 'none') {
                // Don't update score when idle in free flight mode
                droneStats.score = this.previousScore || 0;
            } else {
                this.previousScore = droneStats.score;
            }

            // Prepare comprehensive stats object
            const stats = {
                status: drone.getFlightStatus(),
                score: environment ? environment.getScore() : 0,
                speed: speed,
                altitude: altitude,
                health: Math.ceil(drone.health),
                thrust: thrustPercentage,
                crashes: droneStats.crashes,
                landings: droneStats.landings,
                flightTime: flightTime,
                boostActive: droneStats.boostActive,
                boostAvailable: droneStats.boostAvailable,
                boostCooldown: droneStats.boostCooldown,
                posX: drone.mesh.position.x.toFixed(1),
                posY: drone.mesh.position.y.toFixed(1),
                posZ: drone.mesh.position.z.toFixed(1),
                rotX: drone.mesh.rotation.x,
                rotZ: drone.mesh.rotation.z,
                fps: currentFps,
                environment: environment ? 
                    environment.environmentType.charAt(0).toUpperCase() + 
                    environment.environmentType.slice(1) : 'City'
            };

            // Update HUD panel with all stats
            if (hudPanel) {
                hudPanel.update(stats);
            }
        }
    } catch (error) {
        console.warn("Error updating HUD:", error);
    }
}

// Helper function to safely update element content
function updateElementContent(id, content) {
    const element = document.getElementById(id);
    if (element) {
        if (element.tagName.toLowerCase() === 'span') {
            element.textContent = content;
        } else {
            const span = element.querySelector('span');
            if (span) {
                span.textContent = content;
            }
        }
    }
}

function initializeMinimap() {
    const minimapContainer = document.createElement('div');
    minimapContainer.className = 'minimap-container';
    minimapContainer.style.cssText = `
        position: absolute;
        bottom: 20px;
        right: 20px;
        width: 200px;
        height: 200px;
        background-color: rgba(0, 0, 0, 0.7);
        border-radius: 50%;
        border: 2px solid rgba(255, 255, 255, 0.3);
        overflow: hidden;
        z-index: 1000;
        display: none;
        opacity: 0;
        transition: opacity 0.3s ease;
    `;

    const minimapCanvas = document.createElement('canvas');
    minimapCanvas.id = 'minimap';
    minimapCanvas.width = 200;
    minimapCanvas.height = 200;
    minimapContainer.appendChild(minimapCanvas);

    document.body.appendChild(minimapContainer);

    // Create a clipping mask for circular shape
    const ctx = minimapCanvas.getContext('2d');
    ctx.beginPath();
    ctx.arc(100, 100, 98, 0, Math.PI * 2);
    ctx.clip();
}

function updateMinimap() {
    if (!drone || !drone.mesh) return;

    const canvas = document.getElementById('minimap');
    if (!canvas) return;

    const ctx = canvas.getContext('2d');
    const centerX = canvas.width / 2;
    const centerY = canvas.height / 2;
    const scale = 0.5; // Adjust this value to change the minimap zoom level

    // Clear the canvas with a semi-transparent background
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    ctx.fillStyle = 'rgba(0, 0, 0, 0.7)';
    ctx.fillRect(0, 0, canvas.width, canvas.height);

    // Draw grid lines
    ctx.strokeStyle = 'rgba(255, 255, 255, 0.2)';
    ctx.lineWidth = 1;
    const gridSize = 20;
    
    for (let x = -100; x <= 100; x += gridSize) {
        ctx.beginPath();
        ctx.moveTo(centerX + x * scale, 0);
        ctx.lineTo(centerX + x * scale, canvas.height);
        ctx.stroke();
    }
    
    for (let y = -100; y <= 100; y += gridSize) {
        ctx.beginPath();
        ctx.moveTo(0, centerY + y * scale);
        ctx.lineTo(canvas.width, centerY + y * scale);
        ctx.stroke();
    }

    // Draw checkpoints or mission objectives
    if (mission) {
        // Draw checkpoints
        mission.checkpoints.forEach((checkpoint, index) => {
            const relativeX = (checkpoint.position.x - drone.mesh.position.x) * scale;
            const relativeZ = (checkpoint.position.z - drone.mesh.position.z) * scale;
            
            ctx.beginPath();
            ctx.arc(centerX + relativeX, centerY + relativeZ, 4, 0, Math.PI * 2);
            ctx.fillStyle = checkpoint.userData.passed ? '#ff0000' : '#00ff00';
            ctx.fill();
            
            // Draw checkpoint number
            ctx.fillStyle = '#ffffff';
            ctx.font = '10px Arial';
            ctx.fillText(index + 1, centerX + relativeX + 6, centerY + relativeZ + 4);
        });

        // Draw delivery points
        mission.deliveryPoints.forEach((point, index) => {
            const relativeX = (point.position.x - drone.mesh.position.x) * scale;
            const relativeZ = (point.position.z - drone.mesh.position.z) * scale;
            
            ctx.beginPath();
            ctx.rect(centerX + relativeX - 3, centerY + relativeZ - 3, 6, 6);
            ctx.fillStyle = point.userData.delivered ? '#ff0000' : '#00ffff';
            ctx.fill();
        });
    }

    // Draw drone (player position)
    ctx.save();
    ctx.translate(centerX, centerY);
    ctx.rotate(-drone.mesh.rotation.y);
    
    // Draw drone triangle
    ctx.beginPath();
    ctx.moveTo(0, -6);
    ctx.lineTo(-4, 4);
    ctx.lineTo(4, 4);
    ctx.closePath();
    ctx.fillStyle = '#ffff00';
    ctx.fill();
    
    ctx.restore();
}

function animate() {
    requestAnimationFrame(animate);

    try {
        const now = performance.now();
        const deltaTime = (now - lastTime) / 1000;
        lastTime = now;

        if (controls) controls.update();
        if (drone) drone.update(deltaTime);

        if (drone && drone.mesh) {
            // Check collisions with obstacles
            Physics.checkCollision(drone, obstacles);
            Physics.applyBoundaries(drone, { x: 150, y: 100, z: 150 });
            
            // Check environment interactions
            if (environment) {
                environment.checkGatePass(drone);
                environment.checkCollisions(drone);
                environment.update(deltaTime);
            }

            // Update mission progress
            if (mission) {
                mission.checkProgress(drone);
            }

            // Smoother camera following with look-ahead based on velocity
            const targetCameraPos = new THREE.Vector3();
            targetCameraPos.copy(drone.mesh.position);
            
            // Calculate forward direction based on drone's rotation
            const forward = new THREE.Vector3(0, 0, -1);
            forward.applyQuaternion(drone.mesh.quaternion);
            
            // Add velocity-based look-ahead that respects drone's rotation
            const lookAhead = new THREE.Vector3();
            lookAhead.copy(drone.velocity).multiplyScalar(0.5);
            
            // Calculate camera offset with fixed vertical angle
            const distance = 15; // Base distance from drone
            const heightOffset = 5; // Base height offset
            const angleOffset = Math.PI / 6; // 30-degree angle for camera (adjust as needed)
            
            // Calculate camera position to maintain consistent viewing angle
            const cameraOffset = new THREE.Vector3(
                0,
                Math.sin(angleOffset) * distance + heightOffset,
                Math.cos(angleOffset) * distance
            );
            
            // Rotate the offset based on drone's yaw (y-rotation) only
            const yawRotation = new THREE.Quaternion();
            yawRotation.setFromEuler(new THREE.Euler(0, drone.mesh.rotation.y, 0));
            cameraOffset.applyQuaternion(yawRotation);
            
            // Apply position and offset
            targetCameraPos.add(lookAhead);
            targetCameraPos.add(cameraOffset);
            
            // Smooth camera movement
            camera.position.lerp(targetCameraPos, 0.05);
            
            // Look at point slightly ahead and above drone's position
            const lookAtPoint = drone.mesh.position.clone();
            lookAtPoint.add(forward.multiplyScalar(10)); // Look further ahead
            lookAtPoint.y += 2; // Slightly above drone
            camera.lookAt(lookAtPoint);
        }

        updateHUD();
        updateMinimap(); // Add minimap update
        renderer.render(scene, camera);
    } catch (error) {
        console.error("Error in animation loop:", error);
    }
}

function startGame() {
    if (!selectedDroneType || !selectedEnvironment) {
        alert('Please select both a drone and an environment before starting');
        return;
    }

    console.log("Starting game with:", {
        drone: selectedDroneType,
        environment: selectedEnvironment,
        mission: selectedMission
    });
    
    try {
        gameStartTime = performance.now(); // Reset game start time
        // Initialize and resume audio after user interaction
        if (audioManager) {
            audioManager.resumeAudio().then(() => {
                console.log('Audio system initialized and resumed');
            }).catch(error => {
                console.warn('Error initializing audio:', error);
            });
        }

        // Hide menu first
        const menuElement = document.getElementById('menu');
        if (menuElement) {
            menuElement.classList.add('hidden');
            menuElement.style.display = 'none';
        }
        
        // Create HUD panel if it doesn't exist and show it
        if (!hudPanel) {
            hudPanel = new HUDPanel();
        }
        hudPanel.panel.style.display = 'block';
        
        // Show minimap
        const minimapContainer = document.querySelector('.minimap-container');
        if (minimapContainer) {
            minimapContainer.style.display = 'block';
            // Fade in the minimap
            setTimeout(() => {
                minimapContainer.style.opacity = '1';
            }, 100);
        }
        
        // Change environment first
        if (environment) {
            environment.changeEnvironment(selectedEnvironment);
            obstacles = environment.getObstacles();
        }
        
        // Create and set up drone
        const newDrone = createDrone();
        if (newDrone && newDrone.mesh) {
            // Set initial position
            newDrone.mesh.position.set(0, 5, 0);
            scene.add(newDrone.mesh);
            
            // Store the drone reference
            drone = newDrone;
            
            // Create controls for the drone
            controls = new Controls(drone);
            
            // Reset camera to follow drone
            camera.position.set(0, 10, 20);
            camera.lookAt(drone.mesh.position);
        }
        
        // Start mission (including free flight)
        if (mission) {
            mission.startMission(selectedMission || 'none');
            
            // Show mission instructions
            const missionInstructions = document.getElementById('missionInstructions');
            if (missionInstructions) {
                missionInstructions.style.display = 'block';
                missionInstructions.style.opacity = '1';
            }
        }
        
        // Reset game state
        if (environment) {
            environment.resetScore();
        }
        
        console.log("Game started successfully");
    } catch (error) {
        console.error("Error starting game:", error);
        console.error(error.stack);
    }
}

function setupUIEventListeners() {
    // Drone selection
    document.querySelectorAll('.drone-option').forEach(option => {
        option.addEventListener('click', function(e) {
            e.preventDefault();
            const droneType = this.dataset.drone;
            selectDrone(droneType);
        });
        
        const button = option.querySelector('.drone-select-btn');
        if (button) {
            button.addEventListener('click', function(e) {
                e.preventDefault();
                e.stopPropagation();
                const droneType = this.closest('.drone-option').dataset.drone;
                selectDrone(droneType);
            });
        }
    });

    // Environment selection
    document.querySelectorAll('.environment-option').forEach(option => {
        option.addEventListener('click', function(e) {
            e.preventDefault();
            const envType = this.dataset.env;
            selectEnvironment(envType);
        });
    });

    // Mission selection
    document.querySelectorAll('.mission-option').forEach(option => {
        option.addEventListener('click', function(e) {
            e.preventDefault();
            const missionType = this.dataset.mission;
            selectMission(missionType);
        });
    });

    // Start button
    const startButton = document.querySelector('.start-btn');
    if (startButton) {
        startButton.addEventListener('click', function(e) {
            e.preventDefault();
            startGame();
        });
    }
}

function captureMenuImages() {
    // Create temporary scene for drone captures
    const captureScene = new THREE.Scene();
    captureScene.background = new THREE.Color(0x87CEEB);
    
    // Add lighting for captures
    const captureLight = new THREE.AmbientLight(0xffffff, 1);
    captureScene.add(captureLight);
    
    // Setup camera for captures
    const captureCamera = new THREE.PerspectiveCamera(60, 1, 0.1, 1000); // 1:1 aspect ratio for captures
    
    // Create temporary renderer for captures with smaller size
    const captureRenderer = new THREE.WebGLRenderer({ 
        antialias: true,
        preserveDrawingBuffer: true 
    });
    captureRenderer.setSize(150, 150); // Reduced size for menu images
    document.body.appendChild(captureRenderer.domElement); // Temporarily add to DOM for proper rendering
    captureRenderer.domElement.style.display = 'none'; // Hide the temporary renderer

    // Function to create a colored placeholder image
    const createPlaceholderImage = (color) => {
        const canvas = document.createElement('canvas');
        canvas.width = 150;
        canvas.height = 150;
        const ctx = canvas.getContext('2d');
        ctx.fillStyle = color;
        ctx.fillRect(0, 0, 150, 150);
        return canvas.toDataURL('image/png');
    };
    
    // Capture drone images with fallback colors
    const drones = [
        { type: 'racing', color: '#4CAF50' },
        { type: 'classic', color: '#2196F3' }
    ];
    
    drones.forEach(({ type, color }) => {
        try {
            const tempDrone = new Drone(type);
            captureScene.add(tempDrone.mesh);
            
            tempDrone.mesh.position.set(0, 0, 0);
            tempDrone.mesh.rotation.y = Math.PI / 4;
            captureCamera.position.set(2, 1, 2);
            captureCamera.lookAt(tempDrone.mesh.position);
            
            captureRenderer.render(captureScene, captureCamera);
            const imageUrl = captureRenderer.domElement.toDataURL('image/png');
            
            const droneImg = document.querySelector(`[data-drone="${type}"] .drone-image`);
            if (droneImg) {
                droneImg.src = imageUrl;
            }
            
            captureScene.remove(tempDrone.mesh);
        } catch (error) {
            console.warn(`Failed to generate ${type} drone preview, using placeholder`);
            const droneImg = document.querySelector(`[data-drone="${type}"] .drone-image`);
            if (droneImg) {
                droneImg.src = createPlaceholderImage(color);
            }
        }
    });
    
    // Environment colors and captures
    const environments = [
        { type: 'city', color: '#607D8B' },
        { type: 'neonCity', color: '#9C27B0' },
        { type: 'snow', color: '#E0E0E0' },
        { type: 'desert', color: '#FFA726' },
        { type: 'forest', color: '#388E3C' },
        { type: 'mountains', color: '#795548' }
    ];
    
    environments.forEach(({ type, color }) => {
        try {
            const tempEnvironment = new Environment(captureScene);
            tempEnvironment.changeEnvironment(type);
            
            captureCamera.position.set(30, 20, 30);
            captureCamera.lookAt(0, 0, 0);
            
            captureRenderer.render(captureScene, captureCamera);
            const imageUrl = captureRenderer.domElement.toDataURL('image/png');
            
            const envImg = document.querySelector(`[data-env="${type}"] img`);
            if (envImg) {
                envImg.src = imageUrl;
            }
            
            tempEnvironment.clearEnvironment();
        } catch (error) {
            console.warn(`Failed to generate ${type} environment preview, using placeholder`);
            const envImg = document.querySelector(`[data-env="${type}"] img`);
            if (envImg) {
                envImg.src = createPlaceholderImage(color);
            }
        }
    });
    
    // Mission preview colors
    const missions = [
        { type: 'none', color: '#9E9E9E' },
        { type: 'race', color: '#F44336' },
        { type: 'stunt', color: '#FF9800' },
        { type: 'delivery', color: '#8BC34A' }
    ];
    
    missions.forEach(({ type, color }) => {
        try {
            const tempEnvironment = new Environment(captureScene);
            if (type !== 'none') {
                const tempMission = new Mission(captureScene, tempEnvironment);
                tempMission.startMission(type);
            }
            
            captureCamera.position.set(15, 10, 15);
            captureCamera.lookAt(0, 0, 0);
            
            captureRenderer.render(captureScene, captureCamera);
            const imageUrl = captureRenderer.domElement.toDataURL('image/png');
            
            const missionImg = document.querySelector(`[data-mission="${type}"] img`);
            if (missionImg) {
                missionImg.src = imageUrl;
            }
            
            tempEnvironment.clearEnvironment();
        } catch (error) {
            console.warn(`Failed to generate ${type} mission preview, using placeholder`);
            const missionImg = document.querySelector(`[data-mission="${type}"] img`);
            if (missionImg) {
                missionImg.src = createPlaceholderImage(color);
            }
        }
    });
    
    // Clean up
    document.body.removeChild(captureRenderer.domElement);
    captureRenderer.dispose();
}

function createSoundControls() {
    // Sound controls are now part of the mission panel structure
    const soundToggle = document.querySelector('#soundControls button');
    if (!soundToggle) return;

    let soundEnabled = true;
    soundToggle.addEventListener('click', () => {
        soundEnabled = !soundEnabled;
        soundToggle.innerHTML = soundEnabled ? '🔊' : '🔇';
        
        // Update all audio contexts and sounds immediately
        const audioManagers = [
            drone?.audioManager,
            mission?.audioManager,
            environment?.audioManager
        ].filter(Boolean);
        
        audioManagers.forEach(manager => {
            if (manager) {
                // Set volume to 0 immediately for instant mute
                manager.setMasterVolume(soundEnabled ? 1 : 0);
                
                // Handle audio context state
                if (manager.audioContext) {
                    if (soundEnabled) {
                        manager.audioContext.resume();
                    } else {
                        manager.audioContext.suspend();
                    }
                }
                
                // Stop all active sounds if muting
                if (!soundEnabled && manager.propellerSound) {
                    manager.propellerSound.oscillators?.forEach(osc => {
                        try {
                            osc.stop();
                        } catch (e) {
                            // Ignore if already stopped
                        }
                    });
                }
            }
        });
    });
}

// Initialize and start
init(); 